# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

WM_ANAT_SNR_METADATA = Metadata(
    id="8df82e78354c9dc463da4f9e7e5dd619c20d563a.boutiques",
    name="wm-anat-snr",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class WmAnatSnrOutputs(typing.NamedTuple):
    """
    Output object returned when calling `wm_anat_snr(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_datafile: OutputPathType
    """Output data file containing SNR and associated metrics"""


def wm_anat_snr(
    subject: str,
    output_file: str,
    force: bool = False,
    nerode: int | None = None,
    tmp_dir: str | None = None,
    cleanup: bool = False,
    no_cleanup: bool = False,
    runner: Runner | None = None,
) -> WmAnatSnrOutputs:
    """
    Measures the anatomical SNR in white matter (WM) for quality assurance (QA).
    This is an experimental metric.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: FreeSurfer subject name.
        output_file: Output file.
        force: Force analysis even if output is up-to-date.
        nerode: Number of erosions for the WM mask.
        tmp_dir: Temporary directory.
        cleanup: Delete temporary directory after execution.
        no_cleanup: Do not delete temporary directory after execution.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WmAnatSnrOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(WM_ANAT_SNR_METADATA)
    cargs = []
    cargs.append("wm-anat-snr")
    cargs.extend([
        "--s",
        subject
    ])
    cargs.extend([
        "--o",
        output_file
    ])
    if force:
        cargs.append("--force")
    if nerode is not None:
        cargs.extend([
            "--nerode",
            str(nerode)
        ])
    if tmp_dir is not None:
        cargs.extend([
            "--tmp",
            tmp_dir
        ])
    if cleanup:
        cargs.append("--cleanup")
    if no_cleanup:
        cargs.append("--no-cleanup")
    ret = WmAnatSnrOutputs(
        root=execution.output_file("."),
        output_datafile=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "WM_ANAT_SNR_METADATA",
    "WmAnatSnrOutputs",
    "wm_anat_snr",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_COMPUTE_BIAS_METADATA = Metadata(
    id="37471e0f282e684b7a904fea88a79fdbc35e80b4.boutiques",
    name="mri_compute_bias",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriComputeBiasOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_compute_bias(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """The output volume containing the bias correction result."""


def mri_compute_bias(
    subjects: list[str],
    output_volume: str,
    runner: Runner | None = None,
) -> MriComputeBiasOutputs:
    """
    Compute bias correction volumes for the given subjects and outputs the result to
    a specified volume.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects: List of subjects for which bias correction is calculated.
        output_volume: Output volume where the result will be stored.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriComputeBiasOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_COMPUTE_BIAS_METADATA)
    cargs = []
    cargs.append("mri_compute_bias")
    cargs.extend(subjects)
    cargs.append(output_volume)
    ret = MriComputeBiasOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_COMPUTE_BIAS_METADATA",
    "MriComputeBiasOutputs",
    "mri_compute_bias",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UBER_SKEL_METADATA = Metadata(
    id="d77a80fc9ecd09ed285b440328c53e4f45eb7735.boutiques",
    name="uber_skel",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class UberSkelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `uber_skel(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def uber_skel(
    qt_options: str | None = None,
    no_gui_flag: bool = False,
    print_script: bool = False,
    save_script: str | None = None,
    user_var: list[str] | None = None,
    help_howto_program: bool = False,
    help_: bool = False,
    help_gui: bool = False,
    history: bool = False,
    show_valid_opts: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> UberSkelOutputs:
    """
    Sample uber processing program for initializing user and control variables, with
    options for both GUI and non-GUI modes.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        qt_options: Pass PyQt4 options directly to the GUI.
        no_gui_flag: Run without the GUI.
        print_script: Print the script.
        save_script: Save the script.
        user_var: Initialize user variables. Usage: -uvar <name> <value>.
        help_howto_program: Show programming comments.
        help_: Show help.
        help_gui: Show help for the GUI.
        history: Show history.
        show_valid_opts: Show valid options.
        version: Show version.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UberSkelOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UBER_SKEL_METADATA)
    cargs = []
    cargs.append("uber_skel.py")
    if qt_options is not None:
        cargs.extend([
            "-qt_opts",
            qt_options
        ])
    if no_gui_flag:
        cargs.append("-no_gui")
    if print_script:
        cargs.append("-print_script")
    if save_script is not None:
        cargs.extend([
            "-save_script",
            save_script
        ])
    if user_var is not None:
        cargs.extend([
            "-uvar",
            *user_var
        ])
    if help_howto_program:
        cargs.append("-help_howto_program")
    if help_:
        cargs.append("-help")
    if help_gui:
        cargs.append("-help_gui")
    if history:
        cargs.append("-hist")
    if show_valid_opts:
        cargs.append("-show_valid_opts")
    if version:
        cargs.append("-ver")
    ret = UberSkelOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UBER_SKEL_METADATA",
    "UberSkelOutputs",
    "uber_skel",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FNIRT_METADATA = Metadata(
    id="e595d42b3f6e67fad59b883d737a927f3f3b91ec.boutiques",
    name="fnirt",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class FnirtOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fnirt(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    field_file_outfile: OutputPathType | None
    """File with warp field."""
    fieldcoeff_file_outfile: OutputPathType | None
    """File with field coefficients."""
    jacobian_file_outfile: OutputPathType | None
    """File containing jacobian of the field."""
    log_file_outfile: OutputPathType | None
    """Name of log-file."""
    modulatedref_file_outfile: OutputPathType | None
    """File containing intensity modulated --ref."""
    warped_file_outfile: OutputPathType | None
    """Warped image."""


def fnirt(
    in_file: InputPathType,
    ref_file: InputPathType,
    affine_file: InputPathType | None = None,
    config_file: typing.Literal["T1_2_MNI152_2mm", "FA_2_FMRIB58_1mm"] | None = None,
    field_file: InputPathType | None = None,
    fieldcoeff_file: InputPathType | None = None,
    jacobian_file: InputPathType | None = None,
    log_file: InputPathType | None = None,
    modulatedref_file: str | None = None,
    refmask_file: InputPathType | None = None,
    warped_file: InputPathType | None = None,
    runner: Runner | None = None,
) -> FnirtOutputs:
    """
    FSL non-linear registration.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        in_file: Name of input image.
        ref_file: Name of reference image.
        affine_file: Name of file containing affine transform.
        config_file: 't1_2_mni152_2mm' or 'fa_2_fmrib58_1mm' or file or string.\
            Name of config file specifying command line arguments.
        field_file: file. Name of output file with field.
        fieldcoeff_file: string representing a file. Name of output file with\
            field coefficients.
        jacobian_file: A file. Name of file for writing out the jacobian of the\
            field (for diagnostic or vbm purposes).
        log_file: Name of log-file.
        modulatedref_file: string representing a file. Name of file for writing\
            out intensity modulated --ref (for diagnostic purposes).
        refmask_file: Name of file with mask in reference space.
        warped_file: Name of output-file containing the --in image after it has\
            been warped to the --ref image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FnirtOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FNIRT_METADATA)
    cargs = []
    cargs.append("fnirt")
    if affine_file is not None:
        cargs.append("--aff=" + execution.input_file(affine_file))
    if config_file is not None:
        cargs.append("--config=" + config_file)
    if field_file is not None:
        cargs.append("--fout=" + execution.input_file(field_file))
    if fieldcoeff_file is not None:
        cargs.append("--cout=" + execution.input_file(fieldcoeff_file))
    cargs.append("--in=" + execution.input_file(in_file))
    if jacobian_file is not None:
        cargs.append("--jout=" + execution.input_file(jacobian_file))
    if log_file is not None:
        cargs.append("--logout=" + execution.input_file(log_file))
    if modulatedref_file is not None:
        cargs.append("--refout=" + modulatedref_file)
    cargs.append("--ref=" + execution.input_file(ref_file))
    if refmask_file is not None:
        cargs.append("--refmask=" + execution.input_file(refmask_file))
    if warped_file is not None:
        cargs.append("--iout=" + execution.input_file(warped_file))
    ret = FnirtOutputs(
        root=execution.output_file("."),
        field_file_outfile=execution.output_file(pathlib.Path(field_file).name + ".nii.gz") if (field_file is not None) else None,
        fieldcoeff_file_outfile=execution.output_file(pathlib.Path(fieldcoeff_file).name + ".nii.gz") if (fieldcoeff_file is not None) else None,
        jacobian_file_outfile=execution.output_file(pathlib.Path(jacobian_file).name + ".mat") if (jacobian_file is not None) else None,
        log_file_outfile=execution.output_file(pathlib.Path(log_file).name + ".txt") if (log_file is not None) else None,
        modulatedref_file_outfile=execution.output_file(modulatedref_file + ".nii.gz") if (modulatedref_file is not None) else None,
        warped_file_outfile=execution.output_file(pathlib.Path(warped_file).name + ".nii.gz") if (warped_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FNIRT_METADATA",
    "FnirtOutputs",
    "fnirt",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__GET_AFNI_ORIENT_METADATA = Metadata(
    id="f8dbdeb31f2cb6c6cd4aea993002d6be907c16b0.boutiques",
    name="@GetAfniOrient",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VGetAfniOrientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__get_afni_orient(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_orient_code: OutputPathType
    """File containing the orientation code"""


def v__get_afni_orient(
    infile: InputPathType,
    exploratory: bool = False,
    runner: Runner | None = None,
) -> VGetAfniOrientOutputs:
    """
    Returns the orient code of AFNI datasets.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        infile: Input AFNI dataset (e.g. Hello+orig.HEAD).
        exploratory: Exploratory flag for additional functionalities.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VGetAfniOrientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__GET_AFNI_ORIENT_METADATA)
    cargs = []
    cargs.append("@GetAfniOrient")
    if exploratory:
        cargs.append("-exp")
    cargs.append(execution.input_file(infile))
    ret = VGetAfniOrientOutputs(
        root=execution.output_file("."),
        output_orient_code=execution.output_file(pathlib.Path(infile).name + "_orient_code.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VGetAfniOrientOutputs",
    "V__GET_AFNI_ORIENT_METADATA",
    "v__get_afni_orient",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_SEG2ANNOT_METADATA = Metadata(
    id="4bbdc9d57b691b8f7d4b0333b1d194fbe54c8e5b.boutiques",
    name="mris_seg2annot",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisSeg2annotOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_seg2annot(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    annotation_file: OutputPathType
    """Output custom annotation file."""


def mris_seg2annot(
    surfseg: InputPathType,
    subject: str,
    hemi: str,
    output_annotation: InputPathType,
    colortable: InputPathType | None = None,
    auto_ctab: str | None = None,
    surf: str | None = None,
    debug: bool = False,
    debug_vertex: float | None = None,
    checkopts: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrisSeg2annotOutputs:
    """
    Converts a surface-based segmentation into a custom annotation file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surfseg: Volume-encoded surface segmentation. Values are indices into\
            the color table.
        subject: Subject name.
        hemi: Surface hemifield.
        output_annotation: Output annotation file. E.g., lh.aparc.annot.
        colortable: Color table used to map segmentation index to name and\
            color.
        auto_ctab: Create a random color table and optionally save it.
        surf: Surface name, default is white.
        debug: Turn on debugging.
        debug_vertex: Turn on debugging for vertex.
        checkopts: Don't run anything, just check options and exit.
        version: Print out version and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisSeg2annotOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_SEG2ANNOT_METADATA)
    cargs = []
    cargs.append("mris_seg2annot")
    cargs.extend([
        "--seg",
        execution.input_file(surfseg)
    ])
    if colortable is not None:
        cargs.extend([
            "--ctab",
            execution.input_file(colortable)
        ])
    if auto_ctab is not None:
        cargs.extend([
            "--ctab-auto",
            auto_ctab
        ])
    cargs.extend([
        "--s",
        subject
    ])
    cargs.extend([
        "--h",
        hemi
    ])
    cargs.extend([
        "--o",
        execution.input_file(output_annotation)
    ])
    if surf is not None:
        cargs.extend([
            "--surf",
            surf
        ])
    if debug:
        cargs.append("--debug")
    if debug_vertex is not None:
        cargs.extend([
            "--debug-vertex",
            str(debug_vertex)
        ])
    if checkopts:
        cargs.append("--checkopts")
    if version:
        cargs.append("--version")
    ret = MrisSeg2annotOutputs(
        root=execution.output_file("."),
        annotation_file=execution.output_file(pathlib.Path(output_annotation).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_SEG2ANNOT_METADATA",
    "MrisSeg2annotOutputs",
    "mris_seg2annot",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_DEFORM_METADATA = Metadata(
    id="ea5061ef18cb76bb82c43acc63761667972d145b.boutiques",
    name="mris_deform",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisDeformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_deform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    deformed_surface: OutputPathType
    """Deformed surface file output by mris_deform"""


def mris_deform(
    input_surface: InputPathType,
    input_volume: InputPathType,
    xform: InputPathType,
    output_surface: str,
    runner: Runner | None = None,
) -> MrisDeformOutputs:
    """
    A tool for deforming surface meshes using volumetric information from an
    auxiliary volume.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: Input surface file (e.g., lh.white).
        input_volume: Input volume file (e.g., brain.mgz).
        xform: Transformation file (e.g., talairach.xfm).
        output_surface: Output surface file (e.g., lh.white.deformed).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisDeformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_DEFORM_METADATA)
    cargs = []
    cargs.append("mris_deform")
    cargs.append(execution.input_file(input_surface))
    cargs.append(execution.input_file(input_volume))
    cargs.append(execution.input_file(xform))
    cargs.append(output_surface)
    ret = MrisDeformOutputs(
        root=execution.output_file("."),
        deformed_surface=execution.output_file(output_surface),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_DEFORM_METADATA",
    "MrisDeformOutputs",
    "mris_deform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DMRI_VOX2VOX_METADATA = Metadata(
    id="c170241b240c3ff4893542415ea2cb91ac0b3c6c.boutiques",
    name="dmri_vox2vox",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class DmriVox2voxOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dmri_vox2vox(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dmri_vox2vox(
    input_files: list[InputPathType],
    output_files: list[str],
    input_reference: InputPathType,
    output_reference: InputPathType,
    affine_registration: InputPathType,
    nonlinear_registration: InputPathType,
    input_directory: str | None = None,
    output_directory: str | None = None,
    inverse_nonlinear: bool = False,
    debug: bool = False,
    check_options: bool = False,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DmriVox2voxOutputs:
    """
    Tool for voxel-to-voxel transformations in diffusion MRI data processing.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_files: Input text file(s).
        output_files: Output text file(s), as many as inputs.
        input_reference: Input reference volume.
        output_reference: Output reference volume.
        affine_registration: Affine registration file (.mat) to be applied\
            first.
        nonlinear_registration: Nonlinear registration file (.m3z) to be\
            applied second.
        input_directory: Input directory, if specified, names of input text\
            files are relative to this.
        output_directory: Output directory, if specified, names of output text\
            files are relative to this.
        inverse_nonlinear: Apply inverse of nonlinear warp when --regnl is used.
        debug: Turn on debugging.
        check_options: Check options and exit without running.
        help_: Print out information on how to use this program.
        version: Print out version and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DmriVox2voxOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DMRI_VOX2VOX_METADATA)
    cargs = []
    cargs.append("dmri_vox2vox")
    cargs.extend([
        "--in",
        *[execution.input_file(f) for f in input_files]
    ])
    if input_directory is not None:
        cargs.extend([
            "--indir",
            input_directory
        ])
    cargs.extend([
        "--out",
        *output_files
    ])
    if output_directory is not None:
        cargs.extend([
            "--outdir",
            output_directory
        ])
    cargs.extend([
        "--inref",
        execution.input_file(input_reference)
    ])
    cargs.extend([
        "--outref",
        execution.input_file(output_reference)
    ])
    cargs.extend([
        "--reg",
        execution.input_file(affine_registration)
    ])
    cargs.extend([
        "--regnl",
        execution.input_file(nonlinear_registration)
    ])
    if inverse_nonlinear:
        cargs.append("--invnl")
    if debug:
        cargs.append("--debug")
    if check_options:
        cargs.append("--checkopts")
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    ret = DmriVox2voxOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DMRI_VOX2VOX_METADATA",
    "DmriVox2voxOutputs",
    "dmri_vox2vox",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_HIRES_REGISTER_METADATA = Metadata(
    id="e2724f458ab078f29f7dd2a2e7a3af466b56cbb8.boutiques",
    name="mri_hires_register",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriHiresRegisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_hires_register(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output transform file resulting from high-resolution registration"""


def mri_hires_register(
    hires_labeling: InputPathType,
    input_intensity: InputPathType,
    input_aseg: InputPathType,
    output_xform: InputPathType,
    runner: Runner | None = None,
) -> MriHiresRegisterOutputs:
    """
    A tool for high-resolution registration for Freesurfer images.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        hires_labeling: The high resolution labeling input file.
        input_intensity: The input intensity file.
        input_aseg: The input aseg file.
        output_xform: The output transform file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriHiresRegisterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_HIRES_REGISTER_METADATA)
    cargs = []
    cargs.append("mri_hires_register")
    cargs.append(execution.input_file(hires_labeling))
    cargs.append(execution.input_file(input_intensity))
    cargs.append(execution.input_file(input_aseg))
    cargs.append(execution.input_file(output_xform))
    ret = MriHiresRegisterOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(pathlib.Path(output_xform).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_HIRES_REGISTER_METADATA",
    "MriHiresRegisterOutputs",
    "mri_hires_register",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

XFIBRES_METADATA = Metadata(
    id="00d64d41006871cc5a1d207ddbf5b69d43eb50c8.boutiques",
    name="xfibres",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class XfibresOutputs(typing.NamedTuple):
    """
    Output object returned when calling `xfibres(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_logdir: OutputPathType
    """Log directory where output files are saved"""


def xfibres(
    datafile: InputPathType,
    maskfile: InputPathType,
    bvecs: InputPathType,
    bvals: InputPathType,
    logdir: str | None = None,
    forcedir: bool = False,
    max_fibres: float | None = None,
    model: float | None = None,
    fudge: str | None = None,
    num_jumps: float | None = None,
    num_burnin: float | None = None,
    burnin_noard: float | None = None,
    sampleevery: float | None = None,
    updateproposal: float | None = None,
    seed: str | None = None,
    noard: bool = False,
    allard: bool = False,
    nospat: bool = False,
    nonlinear: bool = False,
    cnonlinear: bool = False,
    rician: bool = False,
    add_f0: bool = False,
    ard_f0: bool = False,
    rmean: float | None = None,
    rstd: float | None = None,
    verbose_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> XfibresOutputs:
    """
    Part of FSL - estimates diffusion parameters for multiple fibres per voxel.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        datafile: Data file (e.g., diffusion data).
        maskfile: Mask file defining brain voxels.
        bvecs: B vectors file.
        bvals: B values file.
        logdir: Log directory (default is logdir).
        forcedir: Use the actual directory name given - i.e., don't add + to\
            make a new directory.
        max_fibres: Maximum number of fibres to fit in each voxel (default 1).
        model: Model to use: 1=deconv. with sticks (default), 2=deconv. with\
            sticks and a range of diffusivities, 3=deconv. with zeppelins.
        fudge: ARD fudge factor.
        num_jumps: Number of jumps to be made by MCMC (default 1250).
        num_burnin: Total number of jumps at start of MCMC to be discarded\
            (default 1000).
        burnin_noard: Number of burnin jumps before ARD is imposed (default 0).
        sampleevery: Number of jumps for each sample (MCMC) (default 25).
        updateproposal: Number of jumps for each update to the proposal density\
            std (MCMC) (default 40).
        seed: Seed for pseudo-random number generator.
        noard: Turn ARD off on all fibres.
        allard: Turn ARD on all fibres.
        nospat: Initialize with tensor, not spatially.
        nonlinear: Initialize with nonlinear fitting.
        cnonlinear: Initialize with constrained nonlinear fitting.
        rician: Use Rician noise modelling.
        add_f0: Add to the model an unattenuated signal compartment.
        ard_f0: Use ARD on F0.
        rmean: Set the prior mean for R of model 3 (default: 0.13 - must be\
            <0.5).
        rstd: Set the prior standard deviation for R of model 3 (default: 0.03).
        verbose_flag: Switch on diagnostic messages.
        help_flag: Display help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `XfibresOutputs`).
    """
    if rmean is not None and not (rmean <= 0.5): 
        raise ValueError(f"'rmean' must be less than x <= 0.5 but was {rmean}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(XFIBRES_METADATA)
    cargs = []
    cargs.append("xfibres")
    cargs.extend([
        "-k",
        execution.input_file(datafile)
    ])
    cargs.extend([
        "-m",
        execution.input_file(maskfile)
    ])
    cargs.extend([
        "-r",
        execution.input_file(bvecs)
    ])
    cargs.extend([
        "-b",
        execution.input_file(bvals)
    ])
    if logdir is not None:
        cargs.extend([
            "--ld",
            logdir
        ])
    if forcedir:
        cargs.append("--forcedir")
    if max_fibres is not None:
        cargs.extend([
            "--nf",
            str(max_fibres)
        ])
    if model is not None:
        cargs.extend([
            "--model",
            str(model)
        ])
    if fudge is not None:
        cargs.extend([
            "--fudge",
            fudge
        ])
    if num_jumps is not None:
        cargs.extend([
            "--nj",
            str(num_jumps)
        ])
    if num_burnin is not None:
        cargs.extend([
            "--bi",
            str(num_burnin)
        ])
    if burnin_noard is not None:
        cargs.extend([
            "--bn",
            str(burnin_noard)
        ])
    if sampleevery is not None:
        cargs.extend([
            "--se",
            str(sampleevery)
        ])
    if updateproposal is not None:
        cargs.extend([
            "--upe",
            str(updateproposal)
        ])
    if seed is not None:
        cargs.extend([
            "--seed",
            seed
        ])
    if noard:
        cargs.append("--noard")
    if allard:
        cargs.append("--allard")
    if nospat:
        cargs.append("--nospat")
    if nonlinear:
        cargs.append("--nonlinear")
    if cnonlinear:
        cargs.append("--cnonlinear")
    if rician:
        cargs.append("--rician")
    if add_f0:
        cargs.append("--f0")
    if ard_f0:
        cargs.append("--ardf0")
    if rmean is not None:
        cargs.extend([
            "--Rmean",
            str(rmean)
        ])
    if rstd is not None:
        cargs.extend([
            "--Rstd",
            str(rstd)
        ])
    if verbose_flag:
        cargs.append("-V")
    if help_flag:
        cargs.append("-h")
    ret = XfibresOutputs(
        root=execution.output_file("."),
        output_logdir=execution.output_file("logdir"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "XFIBRES_METADATA",
    "XfibresOutputs",
    "xfibres",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BET_METADATA = Metadata(
    id="cb7a81b5f775bbef34ecb7d03d21d482a296338c.boutiques",
    name="bet",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class BetOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bet(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """Main default mask output of BET"""
    binary_mask: OutputPathType
    """Binary mask file (from -m option)"""
    overlay_file: OutputPathType
    """Overlaid brain surface onto original image"""
    approx_skull_img: OutputPathType
    """Approximate skull image file"""
    output_vtk_mesh: OutputPathType
    """Mesh in VTK format"""
    skull_mask: OutputPathType
    """Output mask for skull image"""
    out_inskull_mask: OutputPathType
    """The in-skull mask file from betsurf (from -A or -A2)"""
    out_inskull_mesh: OutputPathType
    """The in-skull mesh file from betsurf (from -A or -A2)"""
    out_inskull_off: OutputPathType
    """The in-skull mesh .off file from betsurf (from -A or -A2)"""
    out_outskin_mask: OutputPathType
    """The out-skin mask file from betsurf (from -A or -A2)"""
    out_outskin_mesh: OutputPathType
    """The out-skin mesh file from betsurf (from -A or -A2)"""
    out_outskin_off: OutputPathType
    """The out-skin mesh .off file from betsurf (from -A or -A2)"""
    out_outskull_mask: OutputPathType
    """The out-skull mask file from betsurf (from -A or -A2)"""
    out_outskull_mesh: OutputPathType
    """The out-skull mesh file from betsurf (from -A or -A2)"""
    out_outskull_off: OutputPathType
    """The out-skull mesh .off file from betsurf (from -A or -A2)"""


def bet(
    infile: InputPathType,
    maskfile: str = "img_bet",
    fractional_intensity: float | None = None,
    vg_fractional_intensity: float | None = None,
    center_of_gravity: list[float] | None = None,
    overlay: bool = False,
    binary_mask: bool = False,
    approx_skull: bool = False,
    no_seg_output: bool = False,
    vtk_mesh: bool = False,
    head_radius: float | None = None,
    thresholding: bool = False,
    robust_iters: bool = False,
    residual_optic_cleanup: bool = False,
    reduce_bias: bool = False,
    slice_padding: bool = False,
    whole_set_mask: bool = False,
    additional_surfaces: bool = False,
    additional_surfaces_t2: InputPathType | None = None,
    verbose: bool = False,
    debug: bool = False,
    runner: Runner | None = None,
) -> BetOutputs:
    """
    Automated brain extraction tool for FSL.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        infile: Input image (e.g. img.nii.gz).
        maskfile: Output brain mask (e.g. img_bet.nii.gz).
        fractional_intensity: Fractional intensity threshold (0->1);\
            default=0.5; smaller values give larger brain outline estimates.
        vg_fractional_intensity: Vertical gradient in fractional intensity\
            threshold (-1->1); default=0; positive values give larger brain outline\
            at bottom, smaller at top.
        center_of_gravity: The xyz coordinates of the center of gravity\
            (voxels, not mm) of initial mesh surface. Must have exactly three\
            numerical entries in the list (3-vector).
        overlay: Generate brain surface outline overlaid onto original image.
        binary_mask: Generate binary brain mask.
        approx_skull: Generate rough skull image (not as clean as betsurf).
        no_seg_output: Don't generate segmented brain image output.
        vtk_mesh: Generate brain surface as mesh in .vtk format.
        head_radius: head radius (mm not voxels); initial surface sphere is set\
            to half of this.
        thresholding: Apply thresholding to segmented brain image and mask.
        robust_iters: More robust brain center estimation, by iterating BET\
            with a changing center-of-gravity.
        residual_optic_cleanup: This attempts to cleanup residual eye and optic\
            nerve voxels which bet2 can sometimes leave behind. This can be useful\
            when running SIENA or SIENAX, for example. Various stages involving\
            standard-space masking, morphpological operations and thresholdings are\
            combined to produce a result which can often give better results than\
            just running bet2.
        reduce_bias: This attempts to reduce image bias, and residual neck\
            voxels. This can be useful when running SIENA or SIENAX, for example.\
            Various stages involving FAST segmentation-based bias field removal and\
            standard-space masking are combined to produce a result which can often\
            give better results than just running bet2.
        slice_padding: This can improve the brain extraction if only a few\
            slices are present in the data (i.e., a small field of view in the Z\
            direction). This is achieved by padding the end slices in both\
            directions, copying the end slices several times, running bet2 and then\
            removing the added slices.
        whole_set_mask: This option uses bet2 to determine a brain mask on the\
            basis of the first volume in a 4D data set, and applies this to the\
            whole data set. This is principally intended for use on FMRI data, for\
            example to remove eyeballs. Because it is normally important (in this\
            application) that masking be liberal (ie that there be little risk of\
            cutting out valid brain voxels) the -f threshold is reduced to 0.3, and\
            also the brain mask is "dilated" slightly before being used.
        additional_surfaces: This runs both bet2 and betsurf programs in order\
            to get the additional skull and scalp surfaces created by betsurf. This\
            involves registering to standard space in order to allow betsurf to\
            find the standard space masks it needs.
        additional_surfaces_t2: This is the same as -A except that a T2 image\
            is also input, to further improve the estimated skull and scalp\
            surfaces. As well as carrying out the standard space registration this\
            also registers the T2 to the T1 input image.
        verbose: Switch on diagnostic messages.
        debug: Don't delete temporary intermediate images.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BetOutputs`).
    """
    if fractional_intensity is not None and not (0 <= fractional_intensity <= 1): 
        raise ValueError(f"'fractional_intensity' must be between 0 <= x <= 1 but was {fractional_intensity}")
    if vg_fractional_intensity is not None and not (-1 <= vg_fractional_intensity <= 1): 
        raise ValueError(f"'vg_fractional_intensity' must be between -1 <= x <= 1 but was {vg_fractional_intensity}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(BET_METADATA)
    cargs = []
    cargs.append("bet")
    cargs.append(execution.input_file(infile))
    cargs.append(maskfile)
    if fractional_intensity is not None:
        cargs.extend([
            "-f",
            str(fractional_intensity)
        ])
    if vg_fractional_intensity is not None:
        cargs.extend([
            "-g",
            str(vg_fractional_intensity)
        ])
    if center_of_gravity is not None:
        cargs.extend([
            "-c",
            *map(str, center_of_gravity)
        ])
    if overlay:
        cargs.append("-o")
    if binary_mask:
        cargs.append("-m")
    if approx_skull:
        cargs.append("-s")
    if no_seg_output:
        cargs.append("-n")
    if vtk_mesh:
        cargs.append("-e")
    if head_radius is not None:
        cargs.extend([
            "-r",
            str(head_radius)
        ])
    if thresholding:
        cargs.append("-t")
    if robust_iters:
        cargs.append("-R")
    if residual_optic_cleanup:
        cargs.append("-S")
    if reduce_bias:
        cargs.append("-B")
    if slice_padding:
        cargs.append("-Z")
    if whole_set_mask:
        cargs.append("-F")
    if additional_surfaces:
        cargs.append("-A")
    if additional_surfaces_t2 is not None:
        cargs.extend([
            "-A2",
            execution.input_file(additional_surfaces_t2)
        ])
    if verbose:
        cargs.append("-v")
    if debug:
        cargs.append("-d")
    ret = BetOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(maskfile + ".nii.gz"),
        binary_mask=execution.output_file(maskfile + "_mask.nii.gz"),
        overlay_file=execution.output_file(maskfile + "_overlay.nii.gz"),
        approx_skull_img=execution.output_file(maskfile + "_skull.nii.gz"),
        output_vtk_mesh=execution.output_file(maskfile + "_mesh.vtk"),
        skull_mask=execution.output_file(maskfile + "_skull_mask.nii.gz"),
        out_inskull_mask=execution.output_file(maskfile + "_inskull_mask.nii.gz"),
        out_inskull_mesh=execution.output_file(maskfile + "_inskull_mesh.nii.gz"),
        out_inskull_off=execution.output_file(maskfile + "_inskull_mesh.off"),
        out_outskin_mask=execution.output_file(maskfile + "_outskin_mask.nii.gz"),
        out_outskin_mesh=execution.output_file(maskfile + "_outskin_mesh.nii.gz"),
        out_outskin_off=execution.output_file(maskfile + "_outskin_mesh.off"),
        out_outskull_mask=execution.output_file(maskfile + "_outskull_mask.nii.gz"),
        out_outskull_mesh=execution.output_file(maskfile + "_outskull_mesh.nii.gz"),
        out_outskull_off=execution.output_file(maskfile + "_outskull_mesh.off"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BET_METADATA",
    "BetOutputs",
    "bet",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TKREGISTERFV_METADATA = Metadata(
    id="5ebc1a63857c8578925dcc917e8e3ae5ac3c4e9b.boutiques",
    name="tkregisterfv",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class TkregisterfvOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tkregisterfv(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tkregisterfv(
    reg: InputPathType,
    mov: InputPathType | None = None,
    targ: InputPathType | None = None,
    subject: str | None = None,
    fstarg: InputPathType | None = None,
    sd: str | None = None,
    seg: InputPathType | None = None,
    aseg_flag: bool = False,
    aparc_aseg_flag: bool = False,
    opacity: float | None = None,
    surfs_flag: bool = False,
    pial_surfs_flag: bool = False,
    all_surfs_flag: bool = False,
    no_surfs_flag: bool = False,
    lh_only_flag: bool = False,
    rh_only_flag: bool = False,
    surf: InputPathType | None = None,
    aux_s: InputPathType | None = None,
    plane: str | None = None,
    no_config_flag: bool = False,
    mov2: InputPathType | None = None,
    reg2: InputPathType | None = None,
    mov3: InputPathType | None = None,
    reg3: InputPathType | None = None,
    heat_flag: bool = False,
    regheader_flag: bool = False,
    params: list[float] | None = None,
    flip_x_flag: bool = False,
    flip_y_flag: bool = False,
    flip_z_flag: bool = False,
    fstal: bool = False,
    aux: InputPathType | None = None,
    runner: Runner | None = None,
) -> TkregisterfvOutputs:
    """
    A script that runs freeview with arguments like tkregister, focusing on LTA
    files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        reg: LTA registration file.
        mov: Moving image volume.
        targ: Target image volume.
        subject: Subject identifier for FreeSurfer directories.
        fstarg: Freesurfer target volume instead of orig.mgz.
        sd: FreeSurfer SUBJECTS_DIR environment variable.
        seg: Segmentation volume to load.
        aseg_flag: Load aseg.mgz as segmentation volume.
        aparc_aseg_flag: Load aparc+aseg.mgz as segmentation volume.
        opacity: Set segmentation volume opacity.
        surfs_flag: Load left and right hemisphere white matter surfaces.
        pial_surfs_flag: Load pial surfaces instead of white matter surfaces.
        all_surfs_flag: Load both pial and white matter surfaces.
        no_surfs_flag: Do not load any surfaces.
        lh_only_flag: Load only left hemisphere surface.
        rh_only_flag: Load only right hemisphere surface.
        surf: Explicit path to surface to load.
        aux_s: Explicit path to auxiliary surface to load.
        plane: Plane view: cor, sag, ax.
        no_config_flag: Do not automatically raise transform config window.
        mov2: Provide a second moving image volume.
        reg2: Provide a second registration file.
        mov3: Provide a third moving image volume.
        reg3: Provide a third registration file.
        heat_flag: Use heat map color tables for all volumes.
        regheader_flag: Create registration file assuming the two volumes share\
            a RAS.
        params: Affine matrix parameters: translations in mm, rotations in\
            degrees.
        flip_x_flag: Regheader with rx=180.
        flip_y_flag: Regheader with ry=180.
        flip_z_flag: Regheader with rz=180.
        fstal: Modify the talairach.xfm with subject.
        aux: Load auxiliary volumes with registration.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TkregisterfvOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TKREGISTERFV_METADATA)
    cargs = []
    cargs.append("tkregisterfv")
    if mov is not None:
        cargs.extend([
            "--mov",
            execution.input_file(mov)
        ])
    if targ is not None:
        cargs.extend([
            "--targ",
            execution.input_file(targ)
        ])
    cargs.extend([
        "--reg",
        execution.input_file(reg)
    ])
    if subject is not None:
        cargs.extend([
            "--s",
            subject
        ])
    if fstarg is not None:
        cargs.extend([
            "--fstarg",
            execution.input_file(fstarg)
        ])
    if sd is not None:
        cargs.extend([
            "--sd",
            sd
        ])
    if seg is not None:
        cargs.extend([
            "--seg",
            execution.input_file(seg)
        ])
    if aseg_flag:
        cargs.append("--aseg")
    if aparc_aseg_flag:
        cargs.append("--aparc+aseg")
    if opacity is not None:
        cargs.extend([
            "--opacity",
            str(opacity)
        ])
    if surfs_flag:
        cargs.append("--surfs")
    if pial_surfs_flag:
        cargs.append("--pial-surfs")
    if all_surfs_flag:
        cargs.append("--all-surfs")
    if no_surfs_flag:
        cargs.append("--no-surfs")
    if lh_only_flag:
        cargs.append("--lh-only")
    if rh_only_flag:
        cargs.append("--rh-only")
    if surf is not None:
        cargs.extend([
            "--surf",
            execution.input_file(surf)
        ])
    if aux_s is not None:
        cargs.extend([
            "--aux-surf",
            execution.input_file(aux_s)
        ])
    if plane is not None:
        cargs.extend([
            "--plane",
            plane
        ])
    if no_config_flag:
        cargs.append("--no-config")
    if mov2 is not None:
        cargs.extend([
            "--mov2",
            execution.input_file(mov2)
        ])
    if reg2 is not None:
        cargs.extend([
            "--reg2",
            execution.input_file(reg2)
        ])
    if mov3 is not None:
        cargs.extend([
            "--mov3",
            execution.input_file(mov3)
        ])
    if reg3 is not None:
        cargs.extend([
            "--reg3",
            execution.input_file(reg3)
        ])
    if heat_flag:
        cargs.append("--heat")
    if regheader_flag:
        cargs.append("--regheader")
    if params is not None:
        cargs.extend([
            "--params",
            *map(str, params)
        ])
    if flip_x_flag:
        cargs.append("--flip-x")
    if flip_y_flag:
        cargs.append("--flip-y")
    if flip_z_flag:
        cargs.append("--flip-z")
    if fstal:
        cargs.append("--fstal")
    if aux is not None:
        cargs.extend([
            "--aux",
            execution.input_file(aux)
        ])
    ret = TkregisterfvOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TKREGISTERFV_METADATA",
    "TkregisterfvOutputs",
    "tkregisterfv",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_AI_METADATA = Metadata(
    id="694c77fb3122e7ae7609dc66837caa7834538472.boutiques",
    name="antsAI",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class AntsAiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_ai(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform: OutputPathType
    """The output transform (ITK .mat file)."""


def ants_ai(
    metric: typing.Literal["Mattes[fixedImage,movingImage]", "GC[fixedImage,movingImage]", "MI[fixedImage,movingImage]"],
    transform: typing.Literal["Rigid[gradientStep]", "Affine[gradientStep]", "Similarity[gradientStep]", "AlignGeometricCenters", "AlignCentersOfMass"],
    output: str,
    dimensionality: typing.Literal[2, 3] | None = None,
    align_principal_axes: typing.Literal[0, 1] | None = None,
    align_blobs: typing.Literal["numberOfBlobsToExtract", "[numberOfBlobsToExtract,numberOfBlobsToMatch]"] | None = None,
    search_factor: typing.Literal["searchFactor", "[searchFactor,arcFraction]"] | None = None,
    translation_search_grid: typing.Literal["[stepSize, AxBxC]"] | None = None,
    convergence: typing.Literal["numberOfIterations", "[numberOfIterations,convergenceThreshold,convergenceWindowSize]"] | None = None,
    masks: typing.Literal["fixedImageMask", "[fixedImageMask,movingImageMask]"] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> AntsAiOutputs:
    """
    Program to calculate the optimal linear transform parameters for aligning two
    images.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        metric: These image metrics are available: Mattes: Mattes mutual\
            information (recommended), GC: global correlation, MI: joint histogram\
            mutual information.
        transform: Several transform options are available. For the rigid,\
            affine, and similarity transforms, the gradientStep characterizes the\
            gradient descent optimization. The other two transform types finds the\
            simple translation transform which aligns the specified image feature.
        output: Specify the output transform (output format an ITK .mat file).
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        align_principal_axes: Boolean indicating alignment by principal axes.\
            Alternatively, one can align using blobs.
        align_blobs: Boolean indicating alignment by a set of blobs.
        search_factor: Incremental search factor (in degrees) which will sample\
            the arc fraction around the principal axis or default axis.
        translation_search_grid: Translation search grid in mm, which will\
            translate the moving image in each dimension in increments of the step\
            size.
        convergence: Number of iterations.
        masks: Image masks to limit voxels considered by the metric.
        random_seed: Use a fixed seed for random number generation.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsAiOutputs`).
    """
    if random_seed is not None and not (0 <= random_seed): 
        raise ValueError(f"'random_seed' must be greater than 0 <= x but was {random_seed}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_AI_METADATA)
    cargs = []
    cargs.append("antsAI")
    if dimensionality is not None:
        cargs.extend([
            "--dimensionality",
            str(dimensionality)
        ])
    cargs.extend([
        "-m",
        metric
    ])
    cargs.extend([
        "-t",
        transform
    ])
    if align_principal_axes is not None:
        cargs.extend([
            "-p",
            str(align_principal_axes)
        ])
    if align_blobs is not None:
        cargs.extend([
            "-b",
            align_blobs
        ])
    if search_factor is not None:
        cargs.extend([
            "-s",
            search_factor
        ])
    if translation_search_grid is not None:
        cargs.extend([
            "-g",
            translation_search_grid
        ])
    if convergence is not None:
        cargs.extend([
            "-c",
            convergence
        ])
    if masks is not None:
        cargs.extend([
            "-x",
            masks
        ])
    cargs.extend([
        "-o",
        output
    ])
    if random_seed is not None:
        cargs.extend([
            "--random-seed",
            str(random_seed)
        ])
    if verbose is not None:
        cargs.extend([
            "-v",
            str(verbose)
        ])
    ret = AntsAiOutputs(
        root=execution.output_file("."),
        output_transform=execution.output_file(output + ".mat"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_AI_METADATA",
    "AntsAiOutputs",
    "ants_ai",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_NLFIM_METADATA = Metadata(
    id="b2553c2bf57346747079630757e1d16620ea8119.boutiques",
    name="3dNLfim",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dNlfimOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_nlfim(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    freg_outfile: OutputPathType
    """F-test for significance of the regression"""
    frsqr_outfile: OutputPathType
    """R^2 calculation for regression"""
    fsmax_outfile: OutputPathType
    """Signed maximum signal estimate"""
    ftmax_outfile: OutputPathType
    """Time of signed maximum estimate"""
    fpsmax_outfile: OutputPathType
    """Maximum percentage change estimate"""
    farea_outfile: OutputPathType
    """Area between signal and baseline"""
    fparea_outfile: OutputPathType
    """Percentage area of signal estimate"""
    fscoef_outfile: OutputPathType
    """Signal parameter estimate"""
    fncoef_outfile: OutputPathType
    """Noise parameter estimate"""
    tscoef_outfile: OutputPathType
    """T-test for significance of signal parameter"""
    tncoef_outfile: OutputPathType
    """T-test for significance of noise parameter"""
    bucket_outfile: OutputPathType
    """AFNI 'bucket' dataset"""
    sfit_outfile: OutputPathType
    """Output 3d+time signal model fit"""
    snfit_outfile: OutputPathType
    """Output 3d+time signal+noise fit"""


def v_3d_nlfim(
    input_file: InputPathType,
    signal_model: str,
    noise_model: str,
    runner: Runner | None = None,
) -> V3dNlfimOutputs:
    """
    Nonlinear regression for each voxel of the input AFNI 3d+time data set.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_file: Filename of 3d+time data file for input.
        signal_model: Name of the nonlinear signal model.
        noise_model: Name of the linear noise model.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dNlfimOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_NLFIM_METADATA)
    cargs = []
    cargs.append("3dNLfim")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    cargs.extend([
        "-signal",
        signal_model
    ])
    cargs.extend([
        "-noise",
        noise_model
    ])
    cargs.append("[ADDITIONAL_OPTIONS]")
    ret = V3dNlfimOutputs(
        root=execution.output_file("."),
        freg_outfile=execution.output_file("[FREG].fift"),
        frsqr_outfile=execution.output_file("[FRSQR].fift"),
        fsmax_outfile=execution.output_file("[FSMAX].fift"),
        ftmax_outfile=execution.output_file("[FTMAX].fift"),
        fpsmax_outfile=execution.output_file("[FPSMAX].fift"),
        farea_outfile=execution.output_file("[FAREA].fift"),
        fparea_outfile=execution.output_file("[FPAREA].fift"),
        fscoef_outfile=execution.output_file("[FSCOEF].fift"),
        fncoef_outfile=execution.output_file("[FNCOEF].fift"),
        tscoef_outfile=execution.output_file("[TSCOEF].fitt"),
        tncoef_outfile=execution.output_file("[TNCOEF].fitt"),
        bucket_outfile=execution.output_file("[BUCKET].bucket"),
        sfit_outfile=execution.output_file("[SFIT].sfit"),
        snfit_outfile=execution.output_file("[SNFIT].snfit"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dNlfimOutputs",
    "V_3D_NLFIM_METADATA",
    "v_3d_nlfim",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TCALC_METADATA = Metadata(
    id="f811e9be21119eee4df1778a355c771fb0361c10.boutiques",
    name="tcalc",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class TcalcOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tcalc(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """The output image generated after resampling"""
    original_output_image_file: OutputPathType
    """The original non-resampled output image, if save flag is used"""


def tcalc(
    input_image: InputPathType,
    output_image: str,
    echo_time: float | None = None,
    repetition_time: float | None = None,
    mrpar_file: InputPathType | None = None,
    num_voxel_x: float | None = None,
    num_voxel_y: float | None = None,
    num_voxel_z: float | None = None,
    voxel_size_x: float | None = None,
    voxel_size_y: float | None = None,
    voxel_size_z: float | None = None,
    start_position: float | None = None,
    noise_sigma: float | None = None,
    save_flag: bool = False,
    verbose_flag: bool = False,
    runner: Runner | None = None,
) -> TcalcOutputs:
    """
    Resample a 4D phantom for theoretical calculations.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_image: Input image (4D phantom for theoretical calculations).
        output_image: Output image.
        echo_time: Echo Time (TE) in seconds [e.g., T1-weighted images for 3T\
            TE=0.01 s].
        repetition_time: Repetition Time (TR) in seconds [e.g., T1-weighted\
            images for 3T TR=0.7 s].
        mrpar_file: MRpar File.
        num_voxel_x: Number of Voxels along X (default: phantom).
        num_voxel_y: Number of Voxels along Y (default: phantom).
        num_voxel_z: Number of Voxels along Z (default: phantom).
        voxel_size_x: Size of voxels along X (default: phantom).
        voxel_size_y: Size of voxels along Y (default: phantom).
        voxel_size_z: Size of voxels along Z i.e., number of slices (default:\
            phantom).
        start_position: Starting position of the volume in mm (default = 0mm).
        noise_sigma: Add noise with given sigma (default: 0 i.e., no noise).
        save_flag: Save original non-resample output image.
        verbose_flag: Switch on diagnostic messages.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcalcOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCALC_METADATA)
    cargs = []
    cargs.append("tcalc")
    cargs.append("-i")
    cargs.extend([
        "--input",
        execution.input_file(input_image)
    ])
    cargs.append("-o")
    cargs.extend([
        "--output",
        output_image
    ])
    if echo_time is not None:
        cargs.extend([
            "--te",
            str(echo_time)
        ])
    if repetition_time is not None:
        cargs.extend([
            "--tr",
            str(repetition_time)
        ])
    if mrpar_file is not None:
        cargs.extend([
            "--mrpar",
            execution.input_file(mrpar_file)
        ])
    if num_voxel_x is not None:
        cargs.extend([
            "--nx",
            str(num_voxel_x)
        ])
    if num_voxel_y is not None:
        cargs.extend([
            "--ny",
            str(num_voxel_y)
        ])
    if num_voxel_z is not None:
        cargs.extend([
            "--nz",
            str(num_voxel_z)
        ])
    if voxel_size_x is not None:
        cargs.extend([
            "--dx",
            str(voxel_size_x)
        ])
    if voxel_size_y is not None:
        cargs.extend([
            "--dy",
            str(voxel_size_y)
        ])
    if voxel_size_z is not None:
        cargs.extend([
            "--dz",
            str(voxel_size_z)
        ])
    if start_position is not None:
        cargs.extend([
            "--zstart",
            str(start_position)
        ])
    if noise_sigma is not None:
        cargs.extend([
            "--sigma",
            str(noise_sigma)
        ])
    if save_flag:
        cargs.append("--save")
    if verbose_flag:
        cargs.append("--verbose")
    ret = TcalcOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(output_image),
        original_output_image_file=execution.output_file(output_image + "_original"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TCALC_METADATA",
    "TcalcOutputs",
    "tcalc",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_TNORM_METADATA = Metadata(
    id="2f017fb0846acaa4d9cf83666f160e4cc99d7998.boutiques",
    name="3dTnorm",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dTnormOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_tnorm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_dataset: OutputPathType | None
    """Normalized output dataset"""


def v_3d_tnorm(
    input_dataset: InputPathType,
    prefix: str | None = None,
    norm2: bool = False,
    norm_r: bool = False,
    norm1: bool = False,
    normx: bool = False,
    polort: float | None = None,
    l1fit: bool = False,
    runner: Runner | None = None,
) -> V3dTnormOutputs:
    """
    Normalizes each voxel time series by multiplicative scaling.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_dataset: Input dataset (e.g. data.nii).
        prefix: Prefix for the output dataset.
        norm2: L2 normalize (sum of squares = 1).
        norm_r: Normalize so sum of squares = number of time points.
        norm1: L1 normalize (sum of absolute values = 1).
        normx: Scale so max absolute value = 1 (L_infinity norm).
        polort: Detrend with polynomials of order p before normalizing.
        l1fit: Detrend with L1 regression (L2 is default).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dTnormOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_TNORM_METADATA)
    cargs = []
    cargs.append("3dTnorm")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if norm2:
        cargs.append("-norm2")
    if norm_r:
        cargs.append("-normR")
    if norm1:
        cargs.append("-norm1")
    if normx:
        cargs.append("-normx")
    if polort is not None:
        cargs.extend([
            "-polort",
            str(polort)
        ])
    if l1fit:
        cargs.append("-L1fit")
    cargs.append(execution.input_file(input_dataset))
    ret = V3dTnormOutputs(
        root=execution.output_file("."),
        output_dataset=execution.output_file(prefix + ".nii") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dTnormOutputs",
    "V_3D_TNORM_METADATA",
    "v_3d_tnorm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA = Metadata(
    id="6ce7851c544e610fc5a77e4703824ad332c7d885.boutiques",
    name="WarpTimeSeriesImageMultiTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class WarpTimeSeriesImageMultiTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `warp_time_series_image_multi_transform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_result: OutputPathType
    """The transformed image that is saved as output."""


def warp_time_series_image_multi_transform(
    image_dimension: typing.Literal[3, 4],
    moving_image: InputPathType,
    output_image: InputPathType,
    reference_image: InputPathType,
    transforms: list[str],
    interpolation: typing.Literal["NearestNeighbor", "BSpline"] | None = None,
    runner: Runner | None = None,
) -> WarpTimeSeriesImageMultiTransformOutputs:
    """
    WarpTimeSeriesImageMultiTransform is a tool used to apply a series of
    transformations to a time series image, either forward or reverse, using affine
    transforms and warps.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the input images (3D or 4D).
        moving_image: The image to which the transformation will be applied. It\
            can be a 3D image with vector voxels or a 4D image with scalar voxels.
        output_image: The output image after transformation. It is resampled\
            based on the reference image domain.
        reference_image: The reference image that defines the space into which\
            the input image will be warped.
        transforms: A list of transformation files, such as affine\
            transformation matrices and warps, applied in sequence.
        interpolation: Specifies the type of interpolation to use: Nearest\
            Neighbor or 3rd order B-Spline.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpTimeSeriesImageMultiTransformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA)
    cargs = []
    cargs.append("WarpTimeSeriesImageMultiTransform")
    cargs.append(str(image_dimension))
    cargs.append(execution.input_file(moving_image))
    cargs.append(execution.input_file(output_image))
    cargs.extend([
        "-R",
        execution.input_file(reference_image)
    ])
    cargs.extend(transforms)
    if interpolation is not None:
        cargs.append(interpolation)
    ret = WarpTimeSeriesImageMultiTransformOutputs(
        root=execution.output_file("."),
        output_image_result=execution.output_file(pathlib.Path(output_image).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA",
    "WarpTimeSeriesImageMultiTransformOutputs",
    "warp_time_series_image_multi_transform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TABLE2MAP_METADATA = Metadata(
    id="a1a6b6fb84c32df9b568c86f461d163ba6cbfca6.boutiques",
    name="table2map",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Table2mapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `table2map(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def table2map(
    input_table: InputPathType,
    output_map: str,
    segmentation: InputPathType | None = None,
    parcellation: InputPathType | None = None,
    columns: list[str] | None = None,
    lookup_table: InputPathType | None = None,
    runner: Runner | None = None,
) -> Table2mapOutputs:
    """
    A tool to map data from a table onto an output map, optionally using
    segmentation or parcellation files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_table: Input table.
        output_map: Output map.
        segmentation: Segmentation to map to.
        parcellation: Parcellation to map to.
        columns: Table columns to map. All are included by default.
        lookup_table: Alternative lookup table.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Table2mapOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TABLE2MAP_METADATA)
    cargs = []
    cargs.append("table2map")
    cargs.extend([
        "-t",
        execution.input_file(input_table)
    ])
    cargs.extend([
        "-o",
        output_map
    ])
    if segmentation is not None:
        cargs.extend([
            "-s",
            execution.input_file(segmentation)
        ])
    if parcellation is not None:
        cargs.extend([
            "-p",
            execution.input_file(parcellation)
        ])
    if columns is not None:
        cargs.extend([
            "-c",
            *columns
        ])
    if lookup_table is not None:
        cargs.extend([
            "-l",
            execution.input_file(lookup_table)
        ])
    ret = Table2mapOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TABLE2MAP_METADATA",
    "Table2mapOutputs",
    "table2map",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_ANNOT_DIFF_METADATA = Metadata(
    id="66fc64b92af2b8d25899007f79214682b156e3d3.boutiques",
    name="mris_annot_diff",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisAnnotDiffOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_annot_diff(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mris_annot_diff(
    annot1: InputPathType,
    annot2: InputPathType,
    diff_ctab: bool = False,
    verbose: bool = False,
    runner: Runner | None = None,
) -> MrisAnnotDiffOutputs:
    """
    Compare two surface annotation files. The program works with .annot only.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        annot1: Input .annot file 1.
        annot2: Input .annot file 2.
        diff_ctab: Diff colortable included in .annot.
        verbose: Print details of annotation/colortable differences.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisAnnotDiffOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_ANNOT_DIFF_METADATA)
    cargs = []
    cargs.append("mris_annot_diff")
    cargs.append(execution.input_file(annot1))
    cargs.append(execution.input_file(annot2))
    if diff_ctab:
        cargs.append("--diff-ctab")
    if verbose:
        cargs.append("--verbose")
    ret = MrisAnnotDiffOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_ANNOT_DIFF_METADATA",
    "MrisAnnotDiffOutputs",
    "mris_annot_diff",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MPR2MNI305_METADATA = Metadata(
    id="43f9475eba43e43bb6641a614ad0e97898040bbb.boutiques",
    name="mpr2mni305",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Mpr2mni305Outputs(typing.NamedTuple):
    """
    Output object returned when calling `mpr2mni305(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mpr2mni305(
    mpr_anat: str,
    runner: Runner | None = None,
) -> Mpr2mni305Outputs:
    """
    Tool for transforming MPRAGE dataset to MNI305 coordinate space.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mpr_anat: MPRAGE anatomical input file (e.g., 131).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Mpr2mni305Outputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MPR2MNI305_METADATA)
    cargs = []
    cargs.append("mpr2mni305")
    cargs.append(mpr_anat)
    ret = Mpr2mni305Outputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MPR2MNI305_METADATA",
    "Mpr2mni305Outputs",
    "mpr2mni305",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_POLYFIT_METADATA = Metadata(
    id="71d806c929428c9ef4f179dbd4ad3511aa5ac59a.boutiques",
    name="3dPolyfit",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dPolyfitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_polyfit(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType | None
    """Fitted output dataset"""
    resid_file: OutputPathType | None
    """Residual dataset"""
    coeff_file: OutputPathType | None
    """Coefficient output file"""


def v_3d_polyfit(
    input_dataset: InputPathType,
    poly_order: int | None = None,
    blur: float | None = None,
    median_radius: float | None = None,
    output_prefix: str | None = None,
    resid_prefix: str | None = None,
    coeff_output: str | None = None,
    automask: bool = False,
    mask_dataset: InputPathType | None = None,
    mean_scale: bool = False,
    clip_box: bool = False,
    fit_method: int | None = None,
    base_dataset: InputPathType | None = None,
    verbose: bool = False,
    runner: Runner | None = None,
) -> V3dPolyfitOutputs:
    """
    Fits a polynomial in space to the input dataset and outputs that fitted dataset.
    You can also add your own basis datasets to the fitting mix.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_dataset: Input dataset (e.g., data.nii.gz).
        poly_order: Maximum polynomial order (0..9); [default=3]; [n=0 is the\
            constant 1]; [n=-1 means only use volumes from '-base'].
        blur: Gaussian blur input dataset (inside mask) with FWHM='f' (mm).
        median_radius: Radius (voxels) of preliminary median filter of input;\
            default is no blurring.
        output_prefix: Use 'pp' for prefix of output dataset (the fit); default\
            prefix is 'Polyfit'; use NULL to skip this output.
        resid_prefix: Use 'rr' for the prefix of the residual dataset; default\
            is not to output residuals.
        coeff_output: Save coefficients of fit into text file cc.1D; default is\
            not to save these coefficients.
        automask: Create a mask (a la 3dAutomask).
        mask_dataset: Create a mask from nonzero voxels in 'mset'; default is\
            not to use a mask.
        mean_scale: Scale the mean value of the fit (inside the mask) to 1;\
            probably this option is not useful for anything.
        clip_box: Clip fit values outside the rectilinear box containing the\
            mask to the edge of that box, to avoid weird artifacts.
        fit_method: Set 'mm' to 2 for least squares fit; set it to 1 for L1 fit\
            [default method=2]; [Note that L1 fitting is slower than L2 fitting].
        base_dataset: In addition to the polynomial fit, also use the volumes\
            in dataset 'bb' as extra basis functions.
        verbose: Print fun and useful progress reports.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dPolyfitOutputs`).
    """
    if poly_order is not None and not (-1 <= poly_order <= 9): 
        raise ValueError(f"'poly_order' must be between -1 <= x <= 9 but was {poly_order}")
    if fit_method is not None and not (1 <= fit_method <= 2): 
        raise ValueError(f"'fit_method' must be between 1 <= x <= 2 but was {fit_method}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_POLYFIT_METADATA)
    cargs = []
    cargs.append("3dPolyfit")
    cargs.append(execution.input_file(input_dataset))
    if poly_order is not None:
        cargs.extend([
            "-nord",
            str(poly_order)
        ])
    if blur is not None:
        cargs.extend([
            "-blur",
            str(blur)
        ])
    if median_radius is not None:
        cargs.extend([
            "-mrad",
            str(median_radius)
        ])
    if output_prefix is not None:
        cargs.extend([
            "-prefix",
            output_prefix
        ])
    if resid_prefix is not None:
        cargs.extend([
            "-resid",
            resid_prefix
        ])
    if coeff_output is not None:
        cargs.extend([
            "-1Dcoef",
            coeff_output
        ])
    if automask:
        cargs.append("-automask")
    if mask_dataset is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_dataset)
        ])
    if mean_scale:
        cargs.append("-mone")
    if clip_box:
        cargs.append("-mclip")
    if fit_method is not None:
        cargs.extend([
            "-meth",
            str(fit_method)
        ])
    if base_dataset is not None:
        cargs.extend([
            "-base",
            execution.input_file(base_dataset)
        ])
    if verbose:
        cargs.append("-verb")
    ret = V3dPolyfitOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_prefix + ".nii.gz") if (output_prefix is not None) else None,
        resid_file=execution.output_file(resid_prefix + ".nii.gz") if (resid_prefix is not None) else None,
        coeff_file=execution.output_file(coeff_output + ".1D") if (coeff_output is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dPolyfitOutputs",
    "V_3D_POLYFIT_METADATA",
    "v_3d_polyfit",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

IMAND_METADATA = Metadata(
    id="c48ca42cf72c91b13a658c727dcfb9f75ba40bf0.boutiques",
    name="imand",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ImandOutputs(typing.NamedTuple):
    """
    Output object returned when calling `imand(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """The resulting output image file."""


def imand(
    input_images: list[InputPathType],
    output_image: InputPathType,
    threshold: float | None = None,
    runner: Runner | None = None,
) -> ImandOutputs:
    """
    Image AND operation tool. Only pixels nonzero in all input images (and above the
    threshold, if given) will be output.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_images: Input images to be processed. Multiple input images can\
            be specified.
        output_image: Output image file.
        threshold: Threshold value; only pixels above this value will be\
            output. Optional.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImandOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMAND_METADATA)
    cargs = []
    cargs.append("imand")
    if threshold is not None:
        cargs.extend([
            "--thresh",
            str(threshold)
        ])
    cargs.extend([execution.input_file(f) for f in input_images])
    cargs.append(execution.input_file(output_image))
    ret = ImandOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(pathlib.Path(output_image).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "IMAND_METADATA",
    "ImandOutputs",
    "imand",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1DFFT_METADATA = Metadata(
    id="1b882371fd5c103972dacb7d40e09d3d66d4ad04.boutiques",
    name="1dfft",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dfftOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1dfft(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_fft: OutputPathType
    """Output file with the absolute value of the FFT of the input columns."""


def v_1dfft(
    infile: InputPathType,
    outfile: str,
    ignore: float | None = None,
    use: float | None = None,
    nfft: float | None = None,
    tocx: bool = False,
    fromcx: bool = False,
    hilbert: bool = False,
    nodetrend: bool = False,
    runner: Runner | None = None,
) -> V1dfftOutputs:
    """
    Compute the absolute value of the FFT of input columns from an AFNI 1D file.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        infile: Input .1D file containing an ASCII list of numbers arranged in\
            columns.
        outfile: Output file to store the FFT results.
        ignore: Skip the first 'sss' lines in the input file. [default = no\
            skipping].
        use: Use only 'uuu' lines of the input file. [default = use them all].
        nfft: Set FFT length to 'nnn'. [default = length of data (# of lines\
            used)].
        tocx: Save Re and Im parts of transform in 2 columns.
        fromcx: Convert 2 column complex input into 1 column real output. Note:\
            This will not work if the original data FFT length was an odd number.
        hilbert: When -fromcx is used, the inverse FFT will do the Hilbert\
            transform instead.
        nodetrend: Skip the detrending of the input.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dfftOutputs`).
    """
    if ignore is not None and not (0 <= ignore): 
        raise ValueError(f"'ignore' must be greater than 0 <= x but was {ignore}")
    if use is not None and not (0 <= use): 
        raise ValueError(f"'use' must be greater than 0 <= x but was {use}")
    if nfft is not None and not (1 <= nfft): 
        raise ValueError(f"'nfft' must be greater than 1 <= x but was {nfft}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1DFFT_METADATA)
    cargs = []
    cargs.append("1dfft")
    cargs.append(execution.input_file(infile))
    cargs.append(outfile)
    if ignore is not None:
        cargs.extend([
            "-ignore",
            str(ignore)
        ])
    if use is not None:
        cargs.extend([
            "-use",
            str(use)
        ])
    if nfft is not None:
        cargs.extend([
            "-nfft",
            str(nfft)
        ])
    if tocx:
        cargs.append("-tocx")
    if fromcx:
        cargs.append("-fromcx")
    if hilbert:
        cargs.append("-hilbert")
    if nodetrend:
        cargs.append("-nodetrend")
    ret = V1dfftOutputs(
        root=execution.output_file("."),
        out_fft=execution.output_file(outfile),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dfftOutputs",
    "V_1DFFT_METADATA",
    "v_1dfft",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRHISTMATCH_METADATA = Metadata(
    id="82971ece0a248809e6ac4741b981b24094fbed5d.boutiques",
    name="mrhistmatch",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MrhistmatchConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MrhistmatchOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrhistmatch(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image"""


def mrhistmatch(
    type_: str,
    input_: InputPathType,
    target: InputPathType,
    output: str,
    mask_input: InputPathType | None = None,
    mask_target: InputPathType | None = None,
    bins: int | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrhistmatchConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrhistmatchOutputs:
    """
    Modify the intensities of one image to match the histogram of another.
    
    
    
    References:
    
    * If using inverse contrast normalization for inter-modal (DWI - T1)
    registration:
    Bhushan, C.; Haldar, J. P.; Choi, S.; Joshi, A. A.; Shattuck, D. W. & Leahy,
    R. M. Co-registration and distortion correction of diffusion and anatomical
    images based on inverse contrast normalization. NeuroImage, 2015, 115,
    269-280.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        type_: type of histogram matching to perform; options are:\
            scale,linear,nonlinear.
        input_: the input image to be modified.
        target: the input image from which to derive the target histogram.
        output: the output image.
        mask_input: only generate input histogram based on a specified binary\
            mask image.
        mask_target: only generate target histogram based on a specified binary\
            mask image.
        bins: the number of bins to use to generate the histograms.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrhistmatchOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRHISTMATCH_METADATA)
    cargs = []
    cargs.append("mrhistmatch")
    if mask_input is not None:
        cargs.extend([
            "-mask_input",
            execution.input_file(mask_input)
        ])
    if mask_target is not None:
        cargs.extend([
            "-mask_target",
            execution.input_file(mask_target)
        ])
    if bins is not None:
        cargs.extend([
            "-bins",
            str(bins)
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(type_)
    cargs.append(execution.input_file(input_))
    cargs.append(execution.input_file(target))
    cargs.append(output)
    ret = MrhistmatchOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRHISTMATCH_METADATA",
    "MrhistmatchConfig",
    "MrhistmatchOutputs",
    "mrhistmatch",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

COUNT_METADATA = Metadata(
    id="d518eb3cafe76c3d60b5e88beee05f05e59c9422.boutiques",
    name="count",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class CountOutputs(typing.NamedTuple):
    """
    Output object returned when calling `count(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def count(
    bot: str,
    top: str,
    step: str | None = None,
    seed: float | None = None,
    sseed: str | None = None,
    column: bool = False,
    digits: float | None = None,
    form: str | None = None,
    root: str | None = None,
    sep: str | None = None,
    suffix: str | None = None,
    scale: float | None = None,
    comma: bool = False,
    skipnmodm: str | None = None,
    runner: Runner | None = None,
) -> CountOutputs:
    """
    Numbered copies generator with custom format support and random sequence
    options.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        bot: Starting number or character.
        top: Ending number or character.
        step: Stride step or mode (integer step size, R#, S# or S).
        seed: Seed number for random number generator.
        sseed: Seed string for random number generator.
        column: Write output, one number per line.
        digits: Number of digits to print.
        form: Custom format string for printing the numbers.
        root: String to print before the number.
        sep: Separator character between the numbers.
        suffix: String to print after the number.
        scale: Scale factor to multiply each number.
        comma: Put commas between the outputs.
        skipnmodm: Skip numbers with modulus.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CountOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(COUNT_METADATA)
    cargs = []
    cargs.append("count")
    cargs.append(bot)
    cargs.append(top)
    if step is not None:
        cargs.append(step)
    if seed is not None:
        cargs.extend([
            "-seed",
            str(seed)
        ])
    if sseed is not None:
        cargs.extend([
            "-sseed",
            sseed
        ])
    if column:
        cargs.append("-column")
    if digits is not None:
        cargs.extend([
            "-digits",
            str(digits)
        ])
    if form is not None:
        cargs.extend([
            "-form",
            form
        ])
    if root is not None:
        cargs.extend([
            "-root",
            root
        ])
    if sep is not None:
        cargs.extend([
            "-sep",
            sep
        ])
    if suffix is not None:
        cargs.extend([
            "-suffix",
            suffix
        ])
    if scale is not None:
        cargs.extend([
            "-scale",
            str(scale)
        ])
    if comma:
        cargs.append("-comma")
    if skipnmodm is not None:
        cargs.extend([
            "-skipnmodm",
            skipnmodm
        ])
    ret = CountOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "COUNT_METADATA",
    "CountOutputs",
    "count",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

RESPONSEMEAN_METADATA = Metadata(
    id="a8fd34348625b3b1fa9e865a3af1f57c9158d343.boutiques",
    name="responsemean",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


class ResponsemeanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `responsemean(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_response_file: OutputPathType
    """File containing the averaged tissue response function"""


def responsemean(
    input_response: list[InputPathType],
    output_response: InputPathType,
    legacy: bool = False,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> ResponsemeanOutputs:
    """
    Calculate the mean response function from a set of text files.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_response: Input response functions.
        output_response: Output mean response function.
        legacy: Calculate the mean response function from a set of text files.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ResponsemeanOutputs`).
    """
    if continue_scratch_dir is not None and (len(continue_scratch_dir) != 2): 
        raise ValueError(f"Length of 'continue_scratch_dir' must be 2 but was {len(continue_scratch_dir)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(RESPONSEMEAN_METADATA)
    cargs = []
    cargs.append("responsemean")
    cargs.extend([execution.input_file(f) for f in input_response])
    cargs.append(execution.input_file(output_response))
    if legacy:
        cargs.append("-legacy")
    if nocleanup:
        cargs.append("-nocleanup")
    if scratch_dir is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(scratch_dir)
        ])
    if continue_scratch_dir is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in continue_scratch_dir]
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([
            "-config",
            *config
        ])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    ret = ResponsemeanOutputs(
        root=execution.output_file("."),
        output_response_file=execution.output_file(pathlib.Path(output_response).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RESPONSEMEAN_METADATA",
    "ResponsemeanOutputs",
    "responsemean",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ABIDS_TOOL_METADATA = Metadata(
    id="ecdd7e187a3a330cd02d094f809fb0b75a51fe76.boutiques",
    name="abids_tool",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class AbidsToolOutputs(typing.NamedTuple):
    """
    Output object returned when calling `abids_tool(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def abids_tool(
    input_files: list[InputPathType],
    copy_prefix: list[str] | None = None,
    runner: Runner | None = None,
) -> AbidsToolOutputs:
    """
    A tool to work with BIDS formatted datasets created with dcm2niix_afni or
    dcm2niix, mainly to pull information from the matching JSON file and refit the
    input dataset using 3drefit.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_files: At least one 3d+time dataset in NIFTI format.
        copy_prefix: Copy both the NIFTI dataset(s) and matching .json file(s)\
            to PREFIX. Must have the same number of prefixes as datasets.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AbidsToolOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ABIDS_TOOL_METADATA)
    cargs = []
    cargs.append("abids_tool.py")
    cargs.extend([execution.input_file(f) for f in input_files])
    if copy_prefix is not None:
        cargs.extend([
            "-copy",
            *copy_prefix
        ])
    ret = AbidsToolOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ABIDS_TOOL_METADATA",
    "AbidsToolOutputs",
    "abids_tool",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CP_DICOM_METADATA = Metadata(
    id="e87057efacb16ed69de0523e99719b16d1c3fb64.boutiques",
    name="cp-dicom",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class CpDicomOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cp_dicom(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cp_dicom(
    dicom_dir: str,
    output_dir: str,
    debug: bool = False,
    runner: Runner | None = None,
) -> CpDicomOutputs:
    """
    Copies DICOM files into separate directories for each series based on DICOM
    headers.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        dicom_dir: Directory containing DICOM files.
        output_dir: Output directory where sorted DICOM files will be stored.
        debug: Print additional debug information.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CpDicomOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CP_DICOM_METADATA)
    cargs = []
    cargs.append("cp-dicom")
    cargs.extend([
        "-d",
        dicom_dir
    ])
    cargs.extend([
        "-o",
        output_dir
    ])
    if debug:
        cargs.append("-debug")
    ret = CpDicomOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CP_DICOM_METADATA",
    "CpDicomOutputs",
    "cp_dicom",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_REGISTER_METADATA = Metadata(
    id="01ac3575a4acd8aa600baa6d27fee615f0a862c6.boutiques",
    name="mris_register",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisRegisterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_register(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_surface_file: OutputPathType
    """Output surface file after registration."""
    jacobian_output: OutputPathType | None
    """Jacobian output file if specified."""


def mris_register(
    surf_fname: InputPathType,
    target: InputPathType,
    out_fname: str,
    one_flag: bool = False,
    addframe: str | None = None,
    annot_name: str | None = None,
    curvature_fname: InputPathType | None = None,
    canonical_name: str | None = None,
    inflated: bool = False,
    inflated_name: str | None = None,
    label_file: str | None = None,
    orig_name: str | None = None,
    overlay_values: str | None = None,
    overlay_dir: str | None = None,
    starting_reg_fname: InputPathType | None = None,
    jacobian_fname: str | None = None,
    n_averages: float | None = None,
    adaptive: bool = False,
    l_area: float | None = None,
    l_corr: float | None = None,
    curvature_flag: bool = False,
    l_dist: float | None = None,
    dt_value: float | None = None,
    dt_decrease: float | None = None,
    dt_increase: float | None = None,
    l_external: float | None = None,
    error_ratio: float | None = None,
    initial_flag: bool = False,
    l_laplacian: float | None = None,
    line_min: bool = False,
    momentum: float | None = None,
    max_degrees: float | None = None,
    median: bool = False,
    min_degrees: float | None = None,
    multi_scale: float | None = None,
    n_iterations: float | None = None,
    n_angles: float | None = None,
    neighborhood_size: float | None = None,
    l_nlarea: float | None = None,
    no_curv: bool = False,
    no_normalization: bool = False,
    no_rotation: bool = False,
    no_sulc: bool = False,
    num_surfaces: float | None = None,
    overlay_corr: float | None = None,
    max_passes: float | None = None,
    l_parea: float | None = None,
    remove_negative: float | None = None,
    reverse: bool = False,
    rotate_values: str | None = None,
    registration_file: InputPathType | None = None,
    scale: float | None = None,
    search_flag: bool = False,
    spring_value: float | None = None,
    tolerance: float | None = None,
    topology_flag: bool = False,
    vnum: str | None = None,
    vsmooth: bool = False,
    write_iterations: float | None = None,
    gdiag_no: float | None = None,
    vector_flag: bool = False,
    threads: float | None = None,
    version_flag: bool = False,
    runner: Runner | None = None,
) -> MrisRegisterOutputs:
    """
    This program registers a surface to an average surface template.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surf_fname: Surface to register, often {hemi}.sphere.
        target: The data to register to. A template file for average surface or\
            a single-subject's surface.
        out_fname: Output surface file to capture registration. Example:\
            {hemi}.sphere.reg.
        one_flag: Treats target argument as a single subject's surface not a\
            template file.
        addframe: Add field with location in the atlas.
        annot_name: Zeroes medial wall using annotation.
        curvature_fname: Curvature file name.
        canonical_name: Use surface for canonical properties.
        inflated: Use inflated surface for initial alignment.
        inflated_name: Uses this name for inflated surface for initial\
            alignment.
        label_file: Specify a manual label to align with atlas label.
        orig_name: Use this for original properties.
        overlay_values: Adds a variable to the atlas from overlay file,\
            smoothing it specified number of times.
        overlay_dir: Changes overlay path.
        starting_reg_fname: Start registration with coordinates from file.
        jacobian_fname: Write out jacobian of mapping to the specified file.
        n_averages: Set the number of averages.
        adaptive: Use adaptive time step integration.
        l_area: Sets l_area parameter.
        l_corr: Sets the l_corr parameter.
        curvature_flag: Use smoothwm curvature for final alignment.
        l_dist: Specify distance term.
        dt_value: Sets momentum with specified dt.
        dt_decrease: Set decrease for dt.
        dt_increase: Set increase for dt.
        l_external: Sets l_external parameter.
        error_ratio: Sets error ratio.
        initial_flag: Use initial registration.
        l_laplacian: Sets l_laplacian.
        line_min: Integrate using line minimization.
        momentum: Chooses integration_type = INTEGRATE_MOMENTUM and sets\
            momentum value.
        max_degrees: Set max angle for search.
        median: Uses median normalization (instead of mean).
        min_degrees: Set min angle for search.
        multi_scale: Use multi_scale scales for morphing.
        n_iterations: Sets number of iterations.
        n_angles: Set number of angles/search per scale.
        neighborhood_size: Use neighborhood size specified.
        l_nlarea: Sets l_nlarea parameter.
        no_curv: Do not use smoothwm curvature for final alignment.
        no_normalization: Disables normalization.
        no_rotation: Disables initial rigid alignment.
        no_sulc: Disables initial sulc alignment.
        num_surfaces: Use specified number of surfaces/curvatures for\
            alignment.
        overlay_corr: Sets overlay correlation coefficient.
        max_passes: Limit unfolding to specified passes.
        l_parea: Sets l_parea parameter.
        remove_negative: Remove negative triangles with iterative smoothing if\
            non-zero.
        reverse: Mirror image reverse brain before morphing.
        rotate_values: Rotates brain by specified angles.
        registration_file: Extract rotational components from registration\
            file.
        scale: Scales distances by the specified value.
        search_flag: Integrate with binary search line minimization.
        spring_value: Sets the spring parameter.
        tolerance: Specifies tolerance.
        topology_flag: Preserves the topology of positive area triangles.
        vnum: Set neighborhood parameters.
        vsmooth: Use space/time varying smoothness weighting.
        write_iterations: Set number of write iterations.
        gdiag_no: Supplies a vertex number for diagnostics.
        vector_flag: Prints help for Multiframe mode, listing field names and\
            numbers.
        threads: Set number of OMP threads.
        version_flag: Print the version number.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisRegisterOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_REGISTER_METADATA)
    cargs = []
    cargs.append("mris_register")
    cargs.append(execution.input_file(surf_fname))
    cargs.append(execution.input_file(target))
    cargs.append(out_fname)
    if one_flag:
        cargs.append("-1")
    if addframe is not None:
        cargs.extend([
            "-addframe",
            addframe
        ])
    if annot_name is not None:
        cargs.extend([
            "-annot",
            annot_name
        ])
    if curvature_fname is not None:
        cargs.extend([
            "-C",
            execution.input_file(curvature_fname)
        ])
    if canonical_name is not None:
        cargs.extend([
            "-canon",
            canonical_name
        ])
    if inflated:
        cargs.append("-inflated")
    if inflated_name is not None:
        cargs.extend([
            "-infname",
            inflated_name
        ])
    if label_file is not None:
        cargs.extend([
            "-L",
            label_file
        ])
    if orig_name is not None:
        cargs.extend([
            "-O",
            orig_name
        ])
    if overlay_values is not None:
        cargs.extend([
            "-overlay",
            overlay_values
        ])
    if overlay_dir is not None:
        cargs.extend([
            "-overlay-dir",
            overlay_dir
        ])
    if starting_reg_fname is not None:
        cargs.extend([
            "-sreg",
            execution.input_file(starting_reg_fname)
        ])
    if jacobian_fname is not None:
        cargs.extend([
            "-jacobian",
            jacobian_fname
        ])
    if n_averages is not None:
        cargs.extend([
            "-A",
            str(n_averages)
        ])
    if adaptive:
        cargs.append("-adaptive")
    if l_area is not None:
        cargs.extend([
            "-area",
            str(l_area)
        ])
    if l_corr is not None:
        cargs.extend([
            "-corr",
            str(l_corr)
        ])
    if curvature_flag:
        cargs.append("-curv")
    if l_dist is not None:
        cargs.extend([
            "-dist",
            str(l_dist)
        ])
    if dt_value is not None:
        cargs.extend([
            "-dt",
            str(dt_value)
        ])
    if dt_decrease is not None:
        cargs.extend([
            "-dt_dec",
            str(dt_decrease)
        ])
    if dt_increase is not None:
        cargs.extend([
            "-dt_inc",
            str(dt_increase)
        ])
    if l_external is not None:
        cargs.extend([
            "-E",
            str(l_external)
        ])
    if error_ratio is not None:
        cargs.extend([
            "-error_ratio",
            str(error_ratio)
        ])
    if initial_flag:
        cargs.append("-init")
    if l_laplacian is not None:
        cargs.extend([
            "-lap",
            str(l_laplacian)
        ])
    if line_min:
        cargs.append("-lm")
    if momentum is not None:
        cargs.extend([
            "-M",
            str(momentum)
        ])
    if max_degrees is not None:
        cargs.extend([
            "-max_degrees",
            str(max_degrees)
        ])
    if median:
        cargs.append("-median")
    if min_degrees is not None:
        cargs.extend([
            "-min_degrees",
            str(min_degrees)
        ])
    if multi_scale is not None:
        cargs.extend([
            "-multi_scale",
            str(multi_scale)
        ])
    if n_iterations is not None:
        cargs.extend([
            "-N",
            str(n_iterations)
        ])
    if n_angles is not None:
        cargs.extend([
            "-nangles",
            str(n_angles)
        ])
    if neighborhood_size is not None:
        cargs.extend([
            "-nbrs",
            str(neighborhood_size)
        ])
    if l_nlarea is not None:
        cargs.extend([
            "-nlarea",
            str(l_nlarea)
        ])
    if no_curv:
        cargs.append("-nocurv")
    if no_normalization:
        cargs.append("-nonorm")
    if no_rotation:
        cargs.append("-norot")
    if no_sulc:
        cargs.append("-nosulc")
    if num_surfaces is not None:
        cargs.extend([
            "-nsurfaces",
            str(num_surfaces)
        ])
    if overlay_corr is not None:
        cargs.extend([
            "-ocorr",
            str(overlay_corr)
        ])
    if max_passes is not None:
        cargs.extend([
            "-P",
            str(max_passes)
        ])
    if l_parea is not None:
        cargs.extend([
            "-parea",
            str(l_parea)
        ])
    if remove_negative is not None:
        cargs.extend([
            "-remove_negative",
            str(remove_negative)
        ])
    if reverse:
        cargs.append("-reverse")
    if rotate_values is not None:
        cargs.extend([
            "-rotate",
            rotate_values
        ])
    if registration_file is not None:
        cargs.extend([
            "-reg",
            execution.input_file(registration_file)
        ])
    if scale is not None:
        cargs.extend([
            "-S",
            str(scale)
        ])
    if search_flag:
        cargs.append("-search")
    if spring_value is not None:
        cargs.extend([
            "-spring",
            str(spring_value)
        ])
    if tolerance is not None:
        cargs.extend([
            "-tol",
            str(tolerance)
        ])
    if topology_flag:
        cargs.append("-topology")
    if vnum is not None:
        cargs.extend([
            "-vnum",
            vnum
        ])
    if vsmooth:
        cargs.append("-vsmooth")
    if write_iterations is not None:
        cargs.extend([
            "-W",
            str(write_iterations)
        ])
    if gdiag_no is not None:
        cargs.extend([
            "-V",
            str(gdiag_no)
        ])
    if vector_flag:
        cargs.append("-vector")
    if threads is not None:
        cargs.extend([
            "-threads",
            str(threads)
        ])
    if version_flag:
        cargs.append("--version")
    ret = MrisRegisterOutputs(
        root=execution.output_file("."),
        output_surface_file=execution.output_file(out_fname),
        jacobian_output=execution.output_file(jacobian_fname) if (jacobian_fname is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_REGISTER_METADATA",
    "MrisRegisterOutputs",
    "mris_register",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_COMPUTE_SEG_OVERLAP_METADATA = Metadata(
    id="6a55fd087e217296215136f8bd25bc2fa65bfb23.boutiques",
    name="mri_compute_seg_overlap",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriComputeSegOverlapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_compute_seg_overlap(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_compute_seg_overlap(
    segvol1: InputPathType,
    segvol2: InputPathType,
    log_file: str | None = None,
    mean_log_file: str | None = None,
    std_log_file: str | None = None,
    overall_log_flag: bool = False,
    exclude_cortex_flag: bool = False,
    exclude_wm_flag: bool = False,
    all_labels_flag: bool = False,
    dice_params: str | None = None,
    runner: Runner | None = None,
) -> MriComputeSegOverlapOutputs:
    """
    Compute coefficients of overlap between segmentation volumes.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        segvol1: First segmentation volume.
        segvol2: Second segmentation volume.
        log_file: Log file for individual Dice coefficients for 12 structure\
            pairs, plus mean, std, and 'overall'.
        mean_log_file: Log file for mean Dice.
        std_log_file: Log file for std Dice.
        overall_log_flag: Log file for 'overall' Dice (mean excluding wm, gm,\
            and accumbens).
        exclude_cortex_flag: Exclude cerebral cortex labels from all\
            calculation. (0/1 flag, if nonzero).
        exclude_wm_flag: Exclude cerebral white matter labels from all\
            calculation. (0/1 flag, if nonzero).
        all_labels_flag: Check all labels, not just the predefined main\
            structures.
        dice_params: Standalone way to compute Dice coefficients, using seg1,\
            seg2, ctab, ReportEmpty01, ExcludeId, datfile, and tablefile as\
            additional parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriComputeSegOverlapOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_COMPUTE_SEG_OVERLAP_METADATA)
    cargs = []
    cargs.append("mri_compute_seg_overlap")
    cargs.append(execution.input_file(segvol1))
    cargs.append(execution.input_file(segvol2))
    if log_file is not None:
        cargs.extend([
            "-log",
            log_file
        ])
    if mean_log_file is not None:
        cargs.extend([
            "-mlog",
            mean_log_file
        ])
    if std_log_file is not None:
        cargs.extend([
            "-slog",
            std_log_file
        ])
    if overall_log_flag:
        cargs.append("-olog")
    if exclude_cortex_flag:
        cargs.append("-cortex")
    if exclude_wm_flag:
        cargs.append("-wm")
    if all_labels_flag:
        cargs.append("-all_labels")
    if dice_params is not None:
        cargs.extend([
            "-dice",
            dice_params
        ])
    ret = MriComputeSegOverlapOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_COMPUTE_SEG_OVERLAP_METADATA",
    "MriComputeSegOverlapOutputs",
    "mri_compute_seg_overlap",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA = Metadata(
    id="e4f43efff74611f07e60c00f682c377a411a79c7.boutiques",
    name="antsIntermodalityIntrasubject.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class AntsIntermodalityIntrasubjectShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_intermodality_intrasubject_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transformed_image: OutputPathType
    """Output transformed image after registration."""
    output_transform: OutputPathType
    """Transformation matrix or warp field from the registration."""
    output_warped_image: OutputPathType
    """Output warped image in the template space."""


def ants_intermodality_intrasubject_sh(
    dimension: int,
    anatomical_t1_image: InputPathType,
    scalar_image_to_match: InputPathType,
    anatomical_t1brainmask: InputPathType,
    transform_type: typing.Literal[0, 1, 2, 3],
    t1_to_template_prefix: str,
    output_prefix: str,
    anatomical_reference_image: InputPathType | None = None,
    template_space: str | None = None,
    labels_in_template_space: InputPathType | None = None,
    auxiliary_scalar_images: InputPathType | None = None,
    auxiliary_dt_image: InputPathType | None = None,
    runner: Runner | None = None,
) -> AntsIntermodalityIntrasubjectShOutputs:
    """
    Performs registration between a scalar image and a T1 image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimension: Dimensionality of the image, typically 3 for 3D images.
        anatomical_t1_image: Anatomical T1 image (brain or whole-head) to align\
            to.
        scalar_image_to_match: Scalar image to be matched, such as average\
            BOLD, average DWI, etc.
        anatomical_t1brainmask: Brain mask for the anatomical T1 image, should\
            mask out regions not appearing in the scalar image.
        transform_type: Type of transform: 0=rigid, 1=affine,\
            2=rigid+small_def, 3=affine+small_def.
        t1_to_template_prefix: Prefix for T1 to template transform files.
        output_prefix: Prefix for output files.
        anatomical_reference_image: Anatomical reference image to warp to,\
            often higher resolution than the anatomical T1 image.
        template_space: Template space.
        labels_in_template_space: Labels in the template space.
        auxiliary_scalar_images: Auxiliary scalar images to warp to the\
            template.
        auxiliary_dt_image: Auxiliary DT image to warp to the template.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsIntermodalityIntrasubjectShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA)
    cargs = []
    cargs.append("antsIntermodalityIntrasubject.sh")
    cargs.extend([
        "-d",
        str(dimension)
    ])
    cargs.extend([
        "-r",
        execution.input_file(anatomical_t1_image)
    ])
    if anatomical_reference_image is not None:
        cargs.extend([
            "-R",
            execution.input_file(anatomical_reference_image)
        ])
    cargs.extend([
        "-i",
        execution.input_file(scalar_image_to_match)
    ])
    cargs.extend([
        "-x",
        execution.input_file(anatomical_t1brainmask)
    ])
    cargs.extend([
        "-t",
        str(transform_type)
    ])
    cargs.extend([
        "-w",
        t1_to_template_prefix
    ])
    if template_space is not None:
        cargs.extend([
            "-T",
            template_space
        ])
    cargs.extend([
        "-o",
        output_prefix
    ])
    if labels_in_template_space is not None:
        cargs.extend([
            "-l",
            execution.input_file(labels_in_template_space)
        ])
    if auxiliary_scalar_images is not None:
        cargs.extend([
            "-a",
            execution.input_file(auxiliary_scalar_images)
        ])
    if auxiliary_dt_image is not None:
        cargs.extend([
            "-b",
            execution.input_file(auxiliary_dt_image)
        ])
    ret = AntsIntermodalityIntrasubjectShOutputs(
        root=execution.output_file("."),
        output_transformed_image=execution.output_file(output_prefix + "Transformed.nii.gz"),
        output_transform=execution.output_file(output_prefix + "Transform.mat"),
        output_warped_image=execution.output_file(output_prefix + "Warped.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ANTS_INTERMODALITY_INTRASUBJECT_SH_METADATA",
    "AntsIntermodalityIntrasubjectShOutputs",
    "ants_intermodality_intrasubject_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_DISTANCE_TRANSFORM_METADATA = Metadata(
    id="26094bdcfe452c9a39b8df3e9a0e8773e3b9085f.boutiques",
    name="mris_distance_transform",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisDistanceTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_distance_transform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    result_file: OutputPathType
    """Resulting file after applying the distance transform."""


def mris_distance_transform(
    surface: InputPathType,
    label: InputPathType,
    mode: typing.Literal["signed", "unsigned", "outside"],
    output_file: str,
    anterior: float | None = None,
    posterior: float | None = None,
    divide: float | None = None,
    olabel: bool = False,
    runner: Runner | None = None,
) -> MrisDistanceTransformOutputs:
    """
    Computes the distance transform of a label on the surface.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surface: Surface file for processing.
        label: Label file to apply distance transform.
        mode: Mode of the distance transform, can be 'signed', 'unsigned', or\
            'outside'.
        output_file: Output file for the distance transform results.
        anterior: Only use anteriormost <dist> portion of the label.
        posterior: Only use posteriormost <dist> portion of the label.
        divide: Divide label into <n> units along primary eigendirection.
        olabel: Output label subdivisions.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisDistanceTransformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_DISTANCE_TRANSFORM_METADATA)
    cargs = []
    cargs.append("mris_distance_transform")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(label))
    cargs.append(mode)
    cargs.append(output_file)
    if anterior is not None:
        cargs.extend([
            "-anterior",
            str(anterior)
        ])
    if posterior is not None:
        cargs.extend([
            "-posterior",
            str(posterior)
        ])
    if divide is not None:
        cargs.extend([
            "-divide",
            str(divide)
        ])
    if olabel:
        cargs.append("-olabel")
    ret = MrisDistanceTransformOutputs(
        root=execution.output_file("."),
        result_file=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_DISTANCE_TRANSFORM_METADATA",
    "MrisDistanceTransformOutputs",
    "mris_distance_transform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MKXSUBJREG_METADATA = Metadata(
    id="e782131163109567ec01cead15e999955396aeed.boutiques",
    name="mkxsubjreg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MkxsubjregOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mkxsubjreg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mkxsubjreg(
    srcreg: InputPathType,
    targreg: InputPathType,
    targsubj: str | None = None,
    xfm: str | None = None,
    sd: str | None = None,
    fvol: InputPathType | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MkxsubjregOutputs:
    """
    Creates a new registration matrix that maps from the functional volume of the
    source subject to the orig of the target subject through the talairach
    transform.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        srcreg: Path to the source registration file (srcreg.dat).
        targreg: Path to the target registration file (targreg.dat).
        targsubj: Target subject ID; default is talairach.
        xfm: XFM file name relative to transforms.
        sd: Directory containing subject data; default is env SUBJECTS_DIR.
        fvol: Path to example functional volume.
        help_: Display help information.
        version: Display version information.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MkxsubjregOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MKXSUBJREG_METADATA)
    cargs = []
    cargs.append("mkxsubjreg")
    cargs.extend([
        "--srcreg",
        execution.input_file(srcreg)
    ])
    cargs.extend([
        "--targreg",
        execution.input_file(targreg)
    ])
    if targsubj is not None:
        cargs.extend([
            "--targsubj",
            targsubj
        ])
    if xfm is not None:
        cargs.extend([
            "--xfm",
            xfm
        ])
    if sd is not None:
        cargs.extend([
            "--sd",
            sd
        ])
    if fvol is not None:
        cargs.extend([
            "--fvol",
            execution.input_file(fvol)
        ])
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    ret = MkxsubjregOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MKXSUBJREG_METADATA",
    "MkxsubjregOutputs",
    "mkxsubjreg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_ROBUST_TEMPLATE_METADATA = Metadata(
    id="e6d86749b113688755af2fc89adaf79f55c6735f.boutiques",
    name="mri_robust_template",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriRobustTemplateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_robust_template(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_template: OutputPathType
    """Output template volume (final mean/median image)."""
    output_lta_transform: OutputPathType
    """Output transform file."""
    output_mapped_image: OutputPathType
    """Output mapped and resampled image."""
    output_weights_map: OutputPathType
    """Output weights map (outliers)."""


def mri_robust_template(
    mov_files: list[InputPathType],
    template_file: str,
    sat_value: float | None = None,
    satit_flag: bool = False,
    lta_files: list[str] | None = None,
    mapmov_files: list[str] | None = None,
    mapmovhdr_files: list[str] | None = None,
    weights_files: list[str] | None = None,
    oneminusw_flag: bool = False,
    average_type: int | None = None,
    inittp: int | None = None,
    fixtp_flag: bool = False,
    iscale_flag: bool = False,
    iscaleonly_flag: bool = False,
    iscalein_files: list[str] | None = None,
    iscaleout_files: list[str] | None = None,
    transonly_flag: bool = False,
    affine_flag: bool = False,
    ixforms_files: list[str] | None = None,
    masks_files: list[str] | None = None,
    vox2vox_flag: bool = False,
    leastsquares_flag: bool = False,
    noit_flag: bool = False,
    maxit: int | None = None,
    highit: int | None = None,
    epsit: float | None = None,
    pairmaxit: int | None = None,
    pairepsit: float | None = None,
    subsample: int | None = None,
    nomulti_flag: bool = False,
    floattype_flag: bool = False,
    finalnearest_flag: bool = False,
    doubleprec_flag: bool = False,
    cras_flag: bool = False,
    res_thresh: float | None = None,
    frobnorm_thresh: float | None = None,
    debug_flag: bool = False,
    runner: Runner | None = None,
) -> MriRobustTemplateOutputs:
    """
    Constructs an unbiased robust template for longitudinal volumes using an
    iterative method.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        mov_files: Input movable volumes to be aligned to common mean/median\
            template.
        template_file: Output template volume (final mean/median image).
        sat_value: Set outlier sensitivity manually. Higher values mean less\
            sensitivity.
        satit_flag: Auto-detect good sensitivity (recommended for head or full\
            brain scans).
        lta_files: Output transforms to template (for each input).
        mapmov_files: Output images: map and resample each input to template.
        mapmovhdr_files: Output images: header-adjusted movables (no\
            resampling).
        weights_files: Output weights (outliers) in target space.
        oneminusw_flag: Weights (outlier) map will be inverted (0=outlier), as\
            in earlier versions.
        average_type: Construct template from: 0 Mean, 1 Median (default).
        inittp: Use TP# for spatial init (default random), 0: no init.
        fixtp_flag: Map everything to init TP# (init TP is not resampled).
        iscale_flag: Allow also intensity scaling (default off).
        iscaleonly_flag: Only perform iscale (no transformation, default off).
        iscalein_files: Use initial intensity scales.
        iscaleout_files: Output final intensity scales (will activate\
            --iscale).
        transonly_flag: Find 3 parameter translation only.
        affine_flag: Find 12 parameter affine transform.
        ixforms_files: Use initial transforms (lta) on source ('id'=identity).
        masks_files: Input masks applied to movables.
        vox2vox_flag: Output VOX2VOX lta file (default is RAS2RAS).
        leastsquares_flag: Use least squares instead of robust M-estimator (for\
            testing only).
        noit_flag: Do not iterate, just create first template.
        maxit: Iterate max # times (if #tp>2 default 6, else 5 for 2tp reg.).
        highit: Iterate max # times on highest resolution (default 5).
        epsit: Stop iterations when all transform updates fall below the\
            specified value.
        pairmaxit: Iterate max # times (default 5) for individual pairwise\
            registrations.
        pairepsit: Stop individual pairwise registration iterations when\
            transform updates fall below the specified value.
        subsample: Subsample if dimension > specified value on all axes\
            (default no subsampling).
        nomulti_flag: Do not use multi-resolution (only highest resolution).
        floattype_flag: Convert images to float internally (default: keep input\
            type).
        finalnearest_flag: Use nearest neighbor in final interpolation when\
            creating average.
        doubleprec_flag: Use double precision instead of float internally\
            (large memory usage).
        cras_flag: Center template at average CRAS, instead of average\
            barycenter.
        res_thresh: Volume resolution threshold (default is 0.01 mm).
        frobnorm_thresh: Matrix frobenius norm threshold (default is 0.0001).
        debug_flag: Show debug output (default no debug output).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriRobustTemplateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_ROBUST_TEMPLATE_METADATA)
    cargs = []
    cargs.append("mri_robust_template")
    cargs.extend([
        "-mov",
        *[execution.input_file(f) for f in mov_files]
    ])
    cargs.extend([
        "-template",
        template_file
    ])
    if sat_value is not None:
        cargs.extend([
            "--sat",
            str(sat_value)
        ])
    if satit_flag:
        cargs.append("--satit")
    if lta_files is not None:
        cargs.extend([
            "--lta",
            *lta_files
        ])
    if mapmov_files is not None:
        cargs.extend([
            "--mapmov",
            *mapmov_files
        ])
    if mapmovhdr_files is not None:
        cargs.extend([
            "--mapmovhdr",
            *mapmovhdr_files
        ])
    if weights_files is not None:
        cargs.extend([
            "--weights",
            *weights_files
        ])
    if oneminusw_flag:
        cargs.append("--oneminusw")
    if average_type is not None:
        cargs.extend([
            "--average",
            str(average_type)
        ])
    if inittp is not None:
        cargs.extend([
            "--inittp",
            str(inittp)
        ])
    if fixtp_flag:
        cargs.append("--fixtp")
    if iscale_flag:
        cargs.append("--iscale")
    if iscaleonly_flag:
        cargs.append("--iscaleonly")
    if iscalein_files is not None:
        cargs.extend([
            "--iscalein",
            *iscalein_files
        ])
    if iscaleout_files is not None:
        cargs.extend([
            "--iscaleout",
            *iscaleout_files
        ])
    if transonly_flag:
        cargs.append("--transonly")
    if affine_flag:
        cargs.append("--affine")
    if ixforms_files is not None:
        cargs.extend([
            "--ixforms",
            *ixforms_files
        ])
    if masks_files is not None:
        cargs.extend([
            "--masks",
            *masks_files
        ])
    if vox2vox_flag:
        cargs.append("--vox2vox")
    if leastsquares_flag:
        cargs.append("--leastsquares")
    if noit_flag:
        cargs.append("--noit")
    if maxit is not None:
        cargs.extend([
            "--maxit",
            str(maxit)
        ])
    if highit is not None:
        cargs.extend([
            "--highit",
            str(highit)
        ])
    if epsit is not None:
        cargs.extend([
            "--epsit",
            str(epsit)
        ])
    if pairmaxit is not None:
        cargs.extend([
            "--pairmaxit",
            str(pairmaxit)
        ])
    if pairepsit is not None:
        cargs.extend([
            "--pairepsit",
            str(pairepsit)
        ])
    if subsample is not None:
        cargs.extend([
            "--subsample",
            str(subsample)
        ])
    if nomulti_flag:
        cargs.append("--nomulti")
    if floattype_flag:
        cargs.append("--floattype")
    if finalnearest_flag:
        cargs.append("--finalnearest")
    if doubleprec_flag:
        cargs.append("--doubleprec")
    if cras_flag:
        cargs.append("--cras")
    if res_thresh is not None:
        cargs.extend([
            "--res-thresh",
            str(res_thresh)
        ])
    if frobnorm_thresh is not None:
        cargs.extend([
            "--frobnorm-thresh",
            str(frobnorm_thresh)
        ])
    if debug_flag:
        cargs.append("--debug")
    ret = MriRobustTemplateOutputs(
        root=execution.output_file("."),
        output_template=execution.output_file(template_file),
        output_lta_transform=execution.output_file("[TMP_NAME].lta"),
        output_mapped_image=execution.output_file("[TMP_NAME]_to_template.mgz"),
        output_weights_map=execution.output_file("[TMP_NAME]_weights.mgz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_ROBUST_TEMPLATE_METADATA",
    "MriRobustTemplateOutputs",
    "mri_robust_template",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ADWARP_METADATA = Metadata(
    id="22410e65c1361d9cb6b13a0fd56cc72bae69129b.boutiques",
    name="adwarp",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class AdwarpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `adwarp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    header_output: OutputPathType | None
    """Output dataset header file"""
    brick_output: OutputPathType | None
    """Output dataset brick file"""


def adwarp(
    apar: InputPathType,
    dpar: str,
    prefix: str | None = None,
    dxyz: float | None = None,
    verbose: bool = False,
    force: bool = False,
    resam: str | None = None,
    thr: str | None = None,
    func: str | None = None,
    runner: Runner | None = None,
) -> AdwarpOutputs:
    """
    Resamples a 'data parent' dataset to the grid defined by an 'anat parent'
    dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        apar: Set the anat parent dataset (nonoptional).
        dpar: Set the data parent dataset (nonoptional). dset may contain a\
            sub-brick selector, e.g., -dpar 'dset+orig[2,5,7]'.
        prefix: Set the prefix for the output dataset. Default is the prefix of\
            'dset'.
        dxyz: Set the grid spacing in the output dataset. Default is 1 mm.
        verbose: Print out progress reports.
        force: Write out result even if it means deleting an existing dataset.\
            Default is not to overwrite.
        resam: Set resampling mode for all sub-bricks. Modes: NN (Nearest\
            Neighbor), Li (Linear Interpolation), Cu (Cubic Interpolation), Bk\
            (Blocky Interpolation). Default is Li for all sub-bricks.
        thr: Set resampling mode for threshold sub-bricks. Modes: NN (Nearest\
            Neighbor), Li (Linear Interpolation), Cu (Cubic Interpolation), Bk\
            (Blocky Interpolation).
        func: Set resampling mode for functional sub-bricks. Modes: NN (Nearest\
            Neighbor), Li (Linear Interpolation), Cu (Cubic Interpolation), Bk\
            (Blocky Interpolation).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AdwarpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ADWARP_METADATA)
    cargs = []
    cargs.append("adwarp")
    cargs.extend([
        "-apar",
        execution.input_file(apar)
    ])
    cargs.extend([
        "-dpar",
        dpar
    ])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if dxyz is not None:
        cargs.extend([
            "-dxyz",
            str(dxyz)
        ])
    if verbose:
        cargs.append("-verbose")
    if force:
        cargs.append("-force")
    if resam is not None:
        cargs.extend([
            "-resam",
            resam
        ])
    if thr is not None:
        cargs.extend([
            "-thr",
            thr
        ])
    if func is not None:
        cargs.extend([
            "-func",
            func
        ])
    ret = AdwarpOutputs(
        root=execution.output_file("."),
        header_output=execution.output_file(prefix + ".HEAD") if (prefix is not None) else None,
        brick_output=execution.output_file(prefix + ".BRIK") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ADWARP_METADATA",
    "AdwarpOutputs",
    "adwarp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FS_TEMP_DIR_METADATA = Metadata(
    id="eed642f69e179f132dedff65309afb7749a5e059.boutiques",
    name="fs_temp_dir",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class FsTempDirOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fs_temp_dir(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_directory: OutputPathType
    """The created temporary directory path"""


def fs_temp_dir(
    base_directory: str | None = None,
    scratch: bool = False,
    runner: Runner | None = None,
) -> FsTempDirOutputs:
    """
    Generates and creates an empty temporary directory.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        base_directory: Manually specify base temporary directory.
        scratch: Use /scratch directory if available, but FS_TMPDIR takes\
            priority.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FsTempDirOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FS_TEMP_DIR_METADATA)
    cargs = []
    cargs.append("fs_temp_dir")
    if base_directory is not None:
        cargs.extend([
            "-b",
            base_directory
        ])
    if scratch:
        cargs.append("--scratch")
    ret = FsTempDirOutputs(
        root=execution.output_file("."),
        output_directory=execution.output_file("/tmp/tmp.SF5J66"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FS_TEMP_DIR_METADATA",
    "FsTempDirOutputs",
    "fs_temp_dir",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURF_PROJ_METADATA = Metadata(
    id="2dac5339b324d77b7304374911753059963206c9.boutiques",
    name="surf_proj",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class SurfProjOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surf_proj(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    projected_output: OutputPathType
    """Output of the projection"""
    output_surface: OutputPathType | None
    """Output surface file"""


def surf_proj(
    data: InputPathType,
    surface: InputPathType,
    output_file: str,
    surface_reference: InputPathType | None = None,
    transform: InputPathType | None = None,
    meshspace: str | None = None,
    step_size: float | None = None,
    direction: float | None = None,
    operation: str | None = None,
    surface_output: InputPathType | None = None,
    runner: Runner | None = None,
) -> SurfProjOutputs:
    """
    Projects data onto a surface mesh using specified parameters.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        data: Data to project onto surface.
        surface: Surface file.
        output_file: Output file.
        surface_reference: Surface volume reference (default=same as data).
        transform: Data to surface transform (default=Identity).
        meshspace: Mesh space (default='caret').
        step_size: Average over step (mm - default=1).
        direction: If >0 goes towards brain (default=0 i.e. both directions).
        operation: What to do with values: 'mean' (default), 'max', 'median',\
            'last'.
        surface_output: Output surface file, not ASCII matrix (valid only for\
            scalars).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfProjOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURF_PROJ_METADATA)
    cargs = []
    cargs.append("surf_proj")
    cargs.extend([
        "-data",
        execution.input_file(data)
    ])
    cargs.extend([
        "-surf",
        execution.input_file(surface)
    ])
    cargs.extend([
        "-out",
        output_file
    ])
    if surface_reference is not None:
        cargs.extend([
            "--meshref",
            execution.input_file(surface_reference)
        ])
    if transform is not None:
        cargs.extend([
            "--xfm",
            execution.input_file(transform)
        ])
    if meshspace is not None:
        cargs.extend([
            "--meshspace",
            meshspace
        ])
    if step_size is not None:
        cargs.extend([
            "--step",
            str(step_size)
        ])
    if direction is not None:
        cargs.extend([
            "--direction",
            str(direction)
        ])
    if operation is not None:
        cargs.extend([
            "--operation",
            operation
        ])
    if surface_output is not None:
        cargs.extend([
            "--surfout",
            execution.input_file(surface_output)
        ])
    ret = SurfProjOutputs(
        root=execution.output_file("."),
        projected_output=execution.output_file(output_file),
        output_surface=execution.output_file(pathlib.Path(surface_output).name) if (surface_output is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURF_PROJ_METADATA",
    "SurfProjOutputs",
    "surf_proj",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__FSLABEL2DSET_METADATA = Metadata(
    id="9fe2630c3f97cdc57615a68de7b89685d793afec.boutiques",
    name="@FSlabel2dset",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VFslabel2dsetOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__fslabel2dset(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__fslabel2dset(
    fs_label_file: InputPathType,
    val: float | None = None,
    help_: bool = False,
    echo: bool = False,
    keep_tmp: bool = False,
    runner: Runner | None = None,
) -> VFslabel2dsetOutputs:
    """
    A script to convert a FreeSurfer ASCII label file into a SUMA dataset and a SUMA
    ROI.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        fs_label_file: Specify the ASCII label file from FreeSurfer.
        val: Assign integer VAL to the nodes in FS_LABEL_FILE (Default is 1).
        help_: Display help message.
        echo: Turn echo for debugging.
        keep_tmp: Don't cleanup temp files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VFslabel2dsetOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__FSLABEL2DSET_METADATA)
    cargs = []
    cargs.append("@FSlabel2dset")
    cargs.extend([
        "-fs",
        execution.input_file(fs_label_file)
    ])
    if val is not None:
        cargs.extend([
            "-val",
            str(val)
        ])
    if help_:
        cargs.append("-help")
    if echo:
        cargs.append("-echo")
    if keep_tmp:
        cargs.append("-keep_tmp")
    ret = VFslabel2dsetOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VFslabel2dsetOutputs",
    "V__FSLABEL2DSET_METADATA",
    "v__fslabel2dset",
]
